#ifndef TDIConditionH
#define TDIConditionH

// ==========================================================================
// File:         TDICondition.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons, website),
//               Brian Austin (some source code, documentation)
// Description:  This file defines the TDICondition Component 
//
// "TDx_Input_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Input_Library_Defns.H"
#include "TDx_Input_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDICondition
// Description: The TDICondition component wraps multiple DirectX DICONDITION 
//              structures, which contain type specific information for force 
//              feedback effects that have the DIEFT_CONDITION option set. 
//              
//              It is used to define 'Conditions', which are forces that are 
//              applied in response to the movement or position of a control 
//              device's input mechanisms. 
//              
//              There are four standard types of conditions - friction, 
//              damper, inertia and spring. 
//              
//              Use this component by setting TDIEffect::TypeSpecificParams 
//              to refer to either a single TDICondition component or one 
//              TDICondition component per axis associated with a condition. 
//              
//              To create a condition, use the 
//              TDx_InputDevice::CreateEffect() method and pass one of the 
//              following as the GUID parameter: 
//              
//                  GUID_Spring 
//                  GUID_Damper 
//                  GUID_Inertia 
//                  GUID_Friction 
//              
//              The properties should be interpreted according to the type of 
//              condition the component represents. 
//              
//              To setup this component, first set the 
//              TDICondition::ArraySize property, then set each property 
//              using array indexes, eg: 
//              
//                  // create internal storage for 5 DICONDITION structures 
//              
//                  DICondition1->ArraySize = 5; 
//              
//                  // fill each array with appropriate data 
//              
//                  for (int i=0;i<5;i++) 
//                      /{ 
//                      DICondition1->PositiveCoefficient[i] = 1; 
//                      DICondition1->NegativeCoefficient[i] = -1; 
//                      /} 
//              
//                   
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDICondition : public TDIForce
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDICondition : public TDIForce
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDICondition component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ArraySize
    // Description: The ArraySize property contains the number of internal 
    //              DICONDITION structures wrapped inside the TDICondition 
    //              component. 
    //              By default the ArraySize is initialized to 1, meaning 
    //              that this TDICondition component wraps a single 
    //              DICONDITION structure. To wrap multiple structures, set 
    //              the TDICondition::ArraySize to the total number of 
    //              internal structures desired. 
    // ----------------------------------------------------------------------
    __property dword ArraySize = { read=FGetArraySize, write=FSetArraySize, 
    default=1 }; 

    // ----------------------------------------------------------------------
    // Property:    DeadBand
    // Description: The DeadBand property defines a zone where the condition 
    //              is not active, centered around the position defined by 
    //              TDICondition::Offset. 
    //              
    //              When the control device is positioned within the zone 
    //              defined by Offset + or - DeadBand, the conditional force 
    //              will not be applied. 
    //              
    //              This property can range from 0 to 10,000, representing 
    //              the distance in both directions from the offset that is 
    //              to be defined as the dead zone. 
    // ----------------------------------------------------------------------
    __property long DeadBand[dword pArrayIndex] = { read=FGetDeadBand, 
    write=FSetDeadBand }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDICondition method or 
    //              fget/fset. eg. DI_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DI_OK }; 

    // ----------------------------------------------------------------------
    // Property:    NegativeCoefficient
    // Description: The NegativeCoefficient property defines the coefficient 
    //              constant applied to an axis on the negative side of the 
    //              offset value. 
    //              
    //              This property is ignored and 
    //              TDICondition::PositiveCoefficient is used for both 
    //              positive and negative coefficients if support for 
    //              separate coefficients is not available. 
    //              
    //              The coefficient is used as a multiplier to scale the 
    //              effect and can range from -10,000 to 10,000 where the 
    //              extreme values represent the maximum force output the 
    //              device is capable of producing in either direction. 
    //              
    //              For example, this value may determine how fast a spring 
    //              effect's force would increase as a device moved along an 
    //              axis away from the offset value. 
    // ----------------------------------------------------------------------
    __property long NegativeCoefficient[dword pArrayIndex] = { 
    read=FGetNegativeCoefficient, write=FSetNegativeCoefficient }; 

    // ----------------------------------------------------------------------
    // Property:    NegativeSaturation
    // Description: The NegativeSaturation property defines the negative 
    //              saturation point on an axis. 
    //              
    //              Negative saturation usually represents a point on the 
    //              axis beyond which further movement will not affect the 
    //              force being output by the device. 
    //              
    //              This property is ignored and 
    //              TDICondition::PositiveSaturation is used for both 
    //              positive and negative saturation if support for separate 
    //              saturation values is not available. 
    //              
    //              This value can range from 0 to 10,000, representing the 
    //              distance in the negative direction from 
    //              TDICondition::Offset. 
    // ----------------------------------------------------------------------
    __property dword NegativeSaturation[dword pArrayIndex] = { 
    read=FGetNegativeSaturation, write=FSetNegativeSaturation }; 

    // ----------------------------------------------------------------------
    // Property:    Offset
    // Description: The Offset property defines the offset from the zero or 
    //              centered position of an axis at which the condition 
    //              should apply.   
    //              
    //              This value is generally used only for spring or damper 
    //              effects and represents the center or rest point of a 
    //              spring effect, or the greatest velocity value for which 
    //              damping force should be zero. 
    //              
    //              This value ranges from -10,000 to 10,000 where the 
    //              extreme values represent the maximum travel of the device 
    //              along the axis in either direction. 
    // ----------------------------------------------------------------------
    __property long Offset[dword pArrayIndex] = { read=FGetOffset, 
    write=FSetOffset }; 

    // ----------------------------------------------------------------------
    // Property:    PositiveCoefficient
    // Description: The PositiveCoefficient property defines the coefficient 
    //              constant applied to an axis on the positive side of the 
    //              offset value. 
    //              
    //              This property is used for both positive and negative 
    //              coefficients if support for separate coefficients is not 
    //              available. 
    //              
    //              The coefficient is used as a multiplier to scale the 
    //              effect and can range from -10,000 to 10,000 where the 
    //              extreme values represent the maximum force output the 
    //              device is capable of producing in either direction. 
    //              
    //              For example, this value may determine how fast a spring 
    //              effect's force would increase as a device moved along an 
    //              axis away from the offset value. 
    // ----------------------------------------------------------------------
    __property long PositiveCoefficient[dword pArrayIndex] = { 
    read=FGetPositiveCoefficient, write=FSetPositiveCoefficient }; 

    // ----------------------------------------------------------------------
    // Property:    PositiveSaturation
    // Description: The PositiveSaturation property defines the positive 
    //              saturation point on an axis. 
    //              
    //              This usually represents a point on the axis beyond which 
    //              further movement will not affect the force being output 
    //              by the device. 
    //              
    //              This property is used for both positive and negative 
    //              saturations if support for separate saturation values is 
    //              not available and will be ignored if there is no 
    //              saturation support at all. 
    //              
    //              This value can range from 0 to 10,000, representing the 
    //              distance in the positive direction from 
    //              TDICondition::Offset. 
    // ----------------------------------------------------------------------
    __property dword PositiveSaturation[dword pArrayIndex] = { 
    read=FGetPositiveSaturation, write=FSetPositiveSaturation }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of a single 
    //              internal DICONDITION structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    SizeAll
    // Description: The SizeAll property contains the size in bytes of all 
    //              internal DICONDITION structures. This result is the same 
    //              as multiplying TDICondition::ArraySize and 
    //              TDICondition::Size. 
    // ----------------------------------------------------------------------
    __property dword SizeAll = { read=FGetSizeAll, nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDICondition's internal DICONDITION structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // Params:      pArrayIndex - 
    //                The ArrayIndex parameter determines which internal 
    //                structure needs to be cleared. Correct values range from 
    //                0 to TDICondition::ArraySize-1. 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear( dword pArrayIndex ); 

    // ----------------------------------------------------------------------
    // Method:      ClearAll()
    // Description: The ClearAll() method can be used to clear the contents 
    //              of all the internal DICONDITION structures of the 
    //              TDICondition component. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall ClearAll(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDICondition(TComponent* Owner); 
    virtual __fastcall ~TDICondition(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DICONDITION* Internal_DICONDITION_Ptr = { 
    read=FGetInternal_DICONDITION_Ptr, nodefault }; 

    void __fastcall Internal_DICONDITION_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetArraySize(); 
    void __fastcall FSetArraySize( dword pArraySize ); 
    long __fastcall FGetDeadBand( dword pArrayIndex ); 
    void __fastcall FSetDeadBand( dword pArrayIndex, long pDeadBand ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    long __fastcall FGetNegativeCoefficient( dword pArrayIndex ); 
    void __fastcall FSetNegativeCoefficient( dword pArrayIndex, long 
    pNegativeCoefficient ); 
    dword __fastcall FGetNegativeSaturation( dword pArrayIndex ); 
    void __fastcall FSetNegativeSaturation( dword pArrayIndex, dword 
    pNegativeSaturation ); 
    long __fastcall FGetOffset( dword pArrayIndex ); 
    void __fastcall FSetOffset( dword pArrayIndex, long pOffset ); 
    long __fastcall FGetPositiveCoefficient( dword pArrayIndex ); 
    void __fastcall FSetPositiveCoefficient( dword pArrayIndex, long 
    pPositiveCoefficient ); 
    dword __fastcall FGetPositiveSaturation( dword pArrayIndex ); 
    void __fastcall FSetPositiveSaturation( dword pArrayIndex, dword 
    pPositiveSaturation ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    dword __fastcall FGetSizeAll(); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DICONDITION* __fastcall FGetInternal_DICONDITION_Ptr(); 
    void __fastcall FSetInternal_DICONDITION( dword pArrayIndex, DICONDITION* 
    pDICONDITION ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    dword fArraySize; 
    HRESULT fErrorValue; 
    dword fSize; 
    dword fSizeAll; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DICONDITION* fDICONDITION; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
