#ifndef TDIDataFormatH
#define TDIDataFormatH

// ==========================================================================
// File:         TDIDataFormat.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons, website),
//               Brian Austin (some source code, documentation)
// Description:  This file defines the TDIDataFormat Component 
//
// "TDx_Input_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Input_Library_Defns.H"
#include "TDx_Input_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by TDIDataFormat properties.
#include "TDIObjectDataFormat.H"

// ==========================================================================
// Set:         TDIDataFormatFlags_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TDIDataFormat::Flags property using the BCB Object Inspector. 
//              For detailed descriptions of each flag, see the Flags 
//              property description below. 
// ==========================================================================
enum TDIDataFormat_Flags_Flags
    {
    didf_absaxis,
    didf_relaxis
    };
typedef Set< TDIDataFormat_Flags_Flags, 
             didf_absaxis, 
             didf_relaxis >  TDIDataFormat_Flags_Set;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDIDataFormat
// Description: The TDIDataFormat component wraps the DirectX DIDATAFORMAT 
//              structure, which describes the data format of a particular 
//              input device. 
//              
//              Use this component with the TDx_InputDevice::SetDataFormat() 
//              method to create the data format for a specialized device. In 
//              most cases the predefined data formats for mice, joysticks 
//              and keyboards can be used instead of this component. 
//              
//              This component defines the data format for the device as a 
//              whole. For the data format of a device's individual controls, 
//              eg: the buttons, sliders, x-axis or y-axis of a joystick, 
//              etc., use the TDIObjectDataFormat component. 
//              
//              The data format must be set before a device can be acquired 
//              with TDx_InputDevice::Acquire(). Before a device can be used 
//              it must be aquired and while the device is acquired, the data 
//              format cannot be changed. It should only be neccessary to set 
//              the data format once. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDIDataFormat : public TDIData
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDIDataFormat : public TDIData
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Flags_
    // Description: The Flags property defines flags indicating the mode used 
    //              by the axes of the device. 
    //              The described effect applies when the flag is set. 
    //              
    //              When using predefined data formats, this property may not 
    //              be changed. You can, however, use 
    //              TDx_InputDevice::SetPropery() after defining the data 
    //              format but before acquiring the device to modify this 
    //              setting. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'Flags' property via the BCB Object Inspector. 
    //               For detailed descriptions of each flag, see the Flags 
    //              property description below. 
    // ----------------------------------------------------------------------
    __property TDIDataFormat_Flags_Set Flags_ = { read=FGetFlags_, 
    write=FSetFlags_, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    DataSize
    // Description: The DataSize property defines the data packet size, in 
    //              bytes, returned by the device. 
    //              
    //              This must be a multiple of 4 bytes (1 dword) and be 
    //              larger than the highest offset for retrieving data from 
    //              the object. 
    // ----------------------------------------------------------------------
    __property dword DataSize = { read=FGetDataSize, write=FSetDataSize, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    NumObjs
    // Description: The NumObjs property indicates how many elements the 
    //              TDIDataFormat::ObjectFormats array holds. 
    // ----------------------------------------------------------------------
    __property dword NumObjs = { read=FGetNumObjs, write=FSetNumObjs, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    ObjectFormats
    // Description: The ObjectFormats property references an array of 
    //              TDIObjectDataFormat objects. 
    //              
    //              Each element of the array holds the format of an 
    //              individual device control such as a single button or 
    //              axis. 
    //              Care should be taken not to enter the same information 
    //              into multiple elements or multiple pieces of information 
    //              into the same element. 
    // ----------------------------------------------------------------------
    __property TDIObjectDataFormat* ObjectFormats = { read=FGetObjectFormats, 
    write=FSetObjectFormats, default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    ObjectSize
    // Description: The ObjectSize property defines the size, in bytes, of 
    //              the TDIObjectDataFormat objects contained in the array 
    //              referenced by TDIDataFormat::ObjectFormats. 
    // ----------------------------------------------------------------------
    __property dword ObjectSize = { read=FGetObjectSize, 
    write=FSetObjectSize, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property defines the size, in bytes, of the 
    //              internal structure used by this component. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, 
    default=sizeof(DIDATAFORMAT) }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDIDataFormat component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDIDataFormat method or 
    //              fget/fset. eg. DI_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DI_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Flags
    // Description: The Flags property defines flags indicating the mode used 
    //              by the axes of the device. 
    //              The described effect applies when the flag is set. 
    //              
    //              When using predefined data formats, this property may not 
    //              be changed. You can, however, use 
    //              TDx_InputDevice::SetPropery() after defining the data 
    //              format but before acquiring the device to modify this 
    //              setting. 
    // Flags:       DIDF_ABSAXIS -
    //                Use absolute coordinates to define the position of an 
    //                axis. 
    //                Axis coordinates are returned as a value relative to a 
    //                fixed point established when the device was acquired. 
    //                This flag and DIDF_RELAXIS are cannot both be set. 
    //              DIDF_RELAXIS -
    //                Use relative coordinates to define the position of an 
    //                axis. 
    //                Axis coordinates are returned as an amount of change 
    //                relative to the last value that was returned. 
    //                This flag and DIDF_ABSAXIS cannot both be set. 
    // ----------------------------------------------------------------------
    __property dword Flags = { read=FGetFlags, write=FSetFlags, default=0 }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDIDataFormat's internal DIDATAFORMAT structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Property:    Flags_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the Flags property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* Flags_Strings = { read=FGetFlags_Strings };

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDIDataFormat(TComponent* Owner); 
    virtual __fastcall ~TDIDataFormat(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DIDATAFORMAT* Internal_DIDATAFORMAT_Ptr = { 
    read=FGetInternal_DIDATAFORMAT_Ptr, nodefault }; 

    void __fastcall Internal_DIDATAFORMAT_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Object Inspector 'Flag' Access Methods
    // ----------------------------------------------------------------------
    TDIDataFormat_Flags_Set __fastcall FGetFlags_(); 
    void __fastcall FSetFlags_( TDIDataFormat_Flags_Set pFlags_ ); 

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetDataSize(); 
    void __fastcall FSetDataSize( dword pDataSize ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetFlags(); 
    void __fastcall FSetFlags( dword pFlags ); 
    dword __fastcall FGetNumObjs(); 
    void __fastcall FSetNumObjs( dword pNumObjs ); 
    TDIObjectDataFormat* __fastcall FGetObjectFormats(); 
    void __fastcall FSetObjectFormats( TDIObjectDataFormat* pObjectFormats ); 
    dword __fastcall FGetObjectSize(); 
    void __fastcall FSetObjectSize( dword pObjectSize ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 

    // ----------------------------------------------------------------------
    // Flag_Strings Access Methods
    // ----------------------------------------------------------------------
    TStringList* __fastcall FGetFlags_Strings();

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DIDATAFORMAT* __fastcall FGetInternal_DIDATAFORMAT_Ptr(); 
    void __fastcall FSetInternal_DIDATAFORMAT( DIDATAFORMAT* pDIDATAFORMAT ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    TDIObjectDataFormat* fObjectFormats; 

    // ----------------------------------------------------------------------
    // Flags_Strings Variables
    // ----------------------------------------------------------------------
    TStringList* fFlags_Strings;

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DIDATAFORMAT fDIDATAFORMAT; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
