#ifndef TDIDeviceInstanceH
#define TDIDeviceInstanceH

// ==========================================================================
// File:         TDIDeviceInstance.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons, website),
//               Brian Austin (some source code, documentation)
// Description:  This file defines the TDIDeviceInstance Component 
//
// "TDx_Input_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Input_Library_Defns.H"
#include "TDx_Input_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDIDeviceInstance
// Description: The TDIDeviceInstance component wraps the DirectX 
//              DIDEVICEINSTANCE structure containing information about an 
//              instance of a DirectInput device. 
//              
//              This component is used by the 
//              TDx_InputDevice::GetDeviceInfo() method and the 
//              TDx_Input::OnEnumDevice() callback event. 
//              
//              A device instance is a single device, such as a keyboard, 
//              mouse or joystick. 
//              A single physical device which is capable of multiple forms 
//              of input may be represented as more than one device instance. 
//              An example of this may be something like a keyboard that 
//              incorporates a mouse would be represented as separate 
//              keyboard and mouse instances. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDIDeviceInstance : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDIDeviceInstance : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    DevType
    // Description: The DevType property identifies a device as one of four 
    //              basic types, those being keyboard, mouse, joystick and 
    //              unknown. 
    //              
    //              The three standard types have subtypes that further 
    //              specify the device type. All devices can also be defined 
    //              as using HID (Human Interface Device) protocol. 
    //              
    //              Basic types are defined in the least significant byte, 
    //              subtypes in the next byte and further definitions such as 
    //              HID are contained in the high order word. 
    //              
    //              The possible values are : 
    //              
    //              Basic Types 
    //              
    //              DIDEVTYPE_MOUSE - The device is of the mouse/trackball 
    //              basic type. 
    //              DIDEVTYPE_KEYBOARD - The device is of the keyboard basic 
    //              type. 
    //              DIDEVTYPE_JOYSTICK - The device is of the joystick basic 
    //              type. 
    //              DIDEVTYPE_DEVICE - The device does not fall into any of 
    //              the other categories available. 
    //              
    //              Mouse Subtypes 
    //              
    //              DIDEVTYPEMOUSE_UNKNOWN - The device is a mouse, the 
    //              subtype of which could not be determined. 
    //              DIDEVTYPEMOUSE_TRADITIONAL - The device is a mouse of the 
    //              traditional subtype. 
    //              DIDEVTYPEMOUSE_FINGERSTICK - The device is a mouse of the 
    //              fingerstick subtype. 
    //              DIDEVTYPEMOUSE_TOUCHPAD - The device is a mouse of the 
    //              touchpad subtype. 
    //              DIDEVTYPEMOUSE_TRACKBALL - The device is a mouse of the 
    //              trackball subtype. 
    //              
    //              Keyboard Subtypes 
    //              
    //              DIDEVTYPEKEYBOARD_UNKNOWN - The device is a keyboard, the 
    //              subtype of which could not be determined. 
    //              DIDEVTYPEKEYBOARD_PCXT - The device is a keyboard of the 
    //              IBM PC/XT 83-key subtype. 
    //              DIDEVTYPEKEYBOARD_OLIVETTI - The device is a keyboard of 
    //              the Olivetti 102-key subtytpe. 
    //              DIDEVTYPEKEYBOARD_PCAT - The device is a keyboard of the 
    //              IBM PC/AT 84-key subtype. 
    //              DIDEVTYPEKEYBOARD_PCENH - The device is a keyboard of the 
    //              IBM PC Enhanced 101/102-key or Microsoft Natural subtype. 
    //              DIDEVTYPEKEYBOARD_NOKIA1050 - The device is a keyboard of 
    //              the Nokia 1050 subtype. 
    //              DIDEVTYPEKEYBOARD_NOKIA9140 - The device is a keyboard of 
    //              the Nokia 9140 subtype. 
    //              DIDEVTYPEKEYBOARD_NEC98 - The device is a keyboard of the 
    //              Japanese NEC PC98 subtype. 
    //              DIDEVTYPEKEYBOARD_NEC98LAPTOP - The device is a keyboard 
    //              of the Japanese NEC PC98 laptop subtype. 
    //              DIDEVTYPEKEYBOARD_NEC98106 - The device is a keyboard of 
    //              the Japanese NEC PC98 106-key subtype. 
    //              DIDEVTYPEKEYBOARD_JAPAN106 - The device is a keyboard of 
    //              the Japanese 106-key subtype. 
    //              DIDEVTYPEKEYBOARD_JAPANAX - The device is a keyboard of 
    //              the Japanese AX subtype. 
    //              DIDEVTYPEKEYBOARD_J3100 - The device is a keyboard of the 
    //              Japanese J3100 subtype. 
    //              
    //              Joystick Subtypes 
    //              
    //              DIDEVTYPEJOYSTICK_UNKNOWN - The device is a joystick, the 
    //              subtype of which could not be determined. 
    //              DIDEVTYPEJOYSTICK_TRADITIONAL - The device is a joystick 
    //              of the traditional subtype. 
    //              DIDEVTYPEJOYSTICK_FLIGHTSTICK - The device is a joystick 
    //              of the subtype that is optimized for flight simulation. 
    //              DIDEVTYPEJOYSTICK_GAMEPAD - The device is a joystick 
    //              subtype whose primary purpose is to provide button input. 
    //              DIDEVTYPEJOYSTICK_RUDDER - The device is a joystick 
    //              subtype used for flight simulator style yaw control. 
    //              DIDEVTYPEJOYSTICK_WHEEL - The device is a joystick of the 
    //              steering wheel subtype. 
    //              DIDEVTYPEJOYSTICK_HEADTRACKER - The device is a joystick 
    //              subtype that obtains its input by tracking the movement 
    //              of the user's head. 
    //              
    //              HID 
    //              
    //              DIDEVTYPE_HID - The device uses the Human Interface 
    //              Device (HID) protocol. 
    // ----------------------------------------------------------------------
    __property dword DevType = { read=FGetDevType, write=FSetDevType, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    InstanceName
    // Description: The InstanceName property defines the friendly name for 
    //              this instance of the device. 
    //              
    //              The friendly name will be something like 'Keyboard' or 
    //              'Mouse 1'. 
    // ----------------------------------------------------------------------
    __property AnsiString InstanceName = { read=FGetInstanceName, 
    write=FSetInstanceName, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    ProductName
    // Description: The ProductName property defines the friendly name for 
    //              the device. 
    //              
    //              The friendly name will be something like 'Flightstick' or 
    //              'Super Trackball'. 
    // ----------------------------------------------------------------------
    __property AnsiString ProductName = { read=FGetProductName, 
    write=FSetProductName, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property defines the size, in bytes, of the 
    //              internal structure used by this component. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, 
    default=sizeof(DIDEVICEINSTANCE) }; 

    // ----------------------------------------------------------------------
    // Property:    Usage
    // Description: The Usage property defines the HID usage code. 
    //              A usage code only applies to Human Interface Device's, 
    //              otherwise it will be 0. 
    //              
    //              This property is used in the 
    //              TDx_InputDevice::GetObjectInfo() method after being 
    //              combined with the code contained in 
    //              TDIDeviceInstance::UsagePage via the DIMAKEUSAGEDWORD 
    //              macro. 
    // ----------------------------------------------------------------------
    __property word Usage = { read=FGetUsage, write=FSetUsage, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    UsagePage
    // Description: The UsagePage property defines the HID usage page code. 
    //              A usage page code only applies to Human Interface 
    //              Device's, otherwise it will be 0. 
    //              
    //              This property is used in the 
    //              TDx_InputDevice::GetObjectInfo() method after being 
    //              combined with the code contained in 
    //              TDIDeviceInstance::Usage via the DIMAKEUSAGEDWORD macro. 
    // ----------------------------------------------------------------------
    __property word UsagePage = { read=FGetUsagePage, write=FSetUsagePage, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDIDeviceInstance component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDIDeviceInstance method or 
    //              fget/fset. eg. DI_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DI_OK }; 

    // ----------------------------------------------------------------------
    // Property:    FFDriver
    // Description: The FFDriver property defines the globally unique 
    //              identifier (GUID) of the force feedback driver. 
    //              
    //              The driver manufacturer specifies this identifier. 
    // ----------------------------------------------------------------------
    __property GUID FFDriver = { read=FGetFFDriver, write=FSetFFDriver, 
    nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Instance
    // Description: The Instance property defines a GUID for this instance of 
    //              the device. 
    //              
    //              Instance GUID's are specific to individual computers, so 
    //              instance GUID's from another computer are not 
    //              transferrable. 
    // ----------------------------------------------------------------------
    __property GUID Instance = { read=FGetInstance, write=FSetInstance, 
    nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Product
    // Description: The Product property defines the product GUID. 
    //              
    //              The device manufacturer specifies this identifier. 
    // ----------------------------------------------------------------------
    __property GUID Product = { read=FGetProduct, write=FSetProduct, 
    nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDIDeviceInstance's internal DIDEVICEINSTANCE 
    //              structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDIDeviceInstance(TComponent* Owner); 
    virtual __fastcall ~TDIDeviceInstance(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DIDEVICEINSTANCE* Internal_DIDEVICEINSTANCE_Ptr = { 
    read=FGetInternal_DIDEVICEINSTANCE_Ptr, nodefault }; 

    void __fastcall Internal_DIDEVICEINSTANCE_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetDevType(); 
    void __fastcall FSetDevType( dword pDevType ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    GUID __fastcall FGetFFDriver(); 
    void __fastcall FSetFFDriver( GUID pFFDriver ); 
    GUID __fastcall FGetInstance(); 
    void __fastcall FSetInstance( GUID pInstance ); 
    AnsiString __fastcall FGetInstanceName(); 
    void __fastcall FSetInstanceName( AnsiString pInstanceName ); 
    GUID __fastcall FGetProduct(); 
    void __fastcall FSetProduct( GUID pProduct ); 
    AnsiString __fastcall FGetProductName(); 
    void __fastcall FSetProductName( AnsiString pProductName ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    word __fastcall FGetUsage(); 
    void __fastcall FSetUsage( word pUsage ); 
    word __fastcall FGetUsagePage(); 
    void __fastcall FSetUsagePage( word pUsagePage ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DIDEVICEINSTANCE* __fastcall FGetInternal_DIDEVICEINSTANCE_Ptr(); 
    void __fastcall FSetInternal_DIDEVICEINSTANCE( DIDEVICEINSTANCE* 
    pDIDEVICEINSTANCE ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    GUID fFFDriver; 
    GUID fInstance; 
    GUID fProduct; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DIDEVICEINSTANCE fDIDEVICEINSTANCE; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
