#ifndef TDIDeviceObjectDataH
#define TDIDeviceObjectDataH

// ==========================================================================
// File:         TDIDeviceObjectData.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons, website),
//               Brian Austin (some source code, documentation)
// Description:  This file defines the TDIDeviceObjectData Component 
//
// "TDx_Input_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Input_Library_Defns.H"
#include "TDx_Input_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDIDeviceObjectData
// Description: The TDIDeviceObjectData component wraps multiple DirectX 
//              DIDEVICEOBJECTDATA structures, which contain the raw buffered 
//              device information. 
//              
//              This component is used with the 
//              TDx_InputDevice::GetDeviceData() and 
//              TDx_InputDevice::SendDeviceData() methods. 
//              
//              To setup this component, first set the 
//              TDIDeviceObjectData::ArraySize property, then set each 
//              property using array indexes, eg: 
//              
//                  // create internal storage for 5 DIDEVICEOBJECTDATA 
//              structures 
//              
//                  DIDeviceObjectData->ArraySize = 5; 
//              
//                  // fill each array with appropriate data 
//              
//                  for (int i=0;i<5;i++) 
//                      { 
//                      DIDeviceObjectData1->TimeStamp[i] = 0; 
//                      } 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDIDeviceObjectData : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDIDeviceObjectData : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDIDeviceObjectData component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ArraySize
    // Description: The ArraySize property contains the number of internal 
    //              DIDEVICEOBJECTDATA structures wrapped inside the 
    //              TDIDeviceObjectData component. 
    //              By default the ArraySize is initialized to 1, meaning 
    //              that this TDIDeviceObjectData component wraps a single 
    //              DIDEVICEOBJECTDATA structure. To wrap multiple 
    //              structures, set the TDIDeviceObjectData::ArraySize to the 
    //              total number of internal structures desired. 
    // ----------------------------------------------------------------------
    __property dword ArraySize = { read=FGetArraySize, write=FSetArraySize, 
    default=1 }; 

    // ----------------------------------------------------------------------
    // Property:    Data
    // Description: The Data property defines the data obtained from or be 
    //              sent to the device. 
    //              
    //              This property will define the relative motion for a 
    //              relative mode axis, the axis coordinates for an absolute 
    //              mode axis and the high bit of the low byte will be set 
    //              for a button being depressed or cleared when the button 
    //              is released. 
    // ----------------------------------------------------------------------
    __property dword Data[dword pArrayIndex] = { read=FGetData, 
    write=FSetData }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDIDeviceObjectData method or 
    //              fget/fset. eg. DI_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DI_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Ofs
    // Description: The Ofs property defines different information depending 
    //              on how the TDIDeviceObjectData component is being used. 
    //              
    //              For TDx_InputDevice::GetDeviceData() it defines an offset 
    //              into the data format of the device. When accessing one of 
    //              the 3 basic device types, this property should be one of 
    //              the mouse, keyboard or joystick device constants. 
    //              
    //              For TDx_InputDevice::SendDeviceData() it defines the 
    //              instance ID of the object as defined in 
    //              TDIDeviceObjectInstance::Type. 
    // ----------------------------------------------------------------------
    __property dword Ofs[dword pArrayIndex] = { read=FGetOfs, write=FSetOfs 
    }; 

    // ----------------------------------------------------------------------
    // Property:    Sequence
    // Description: The Sequence property defines the sequence number of the 
    //              data retrieval. 
    //              
    //              This property can be used to chronologically sort data 
    //              returned from multiple devices by 
    //              TDx_InputDevice::GetDeviceData() calls. 
    //              
    //              When calling TDx_InputDevice::SendDeviceData(), this 
    //              property must be 0. 
    //              
    //              Sequence numbers can wrap around, so the 
    //              DISEQUENCE_COMPARE macro should be used to perform 
    //              sequence comparisons safely. 
    //              
    //              DISEQUENCE_COMPARE(Val1, cmp, Val2) 
    //              
    //              where cmp can be ==, !=, <, >, <=, or >=. 
    //              
    //              returns nonzero if (val1 cmp Val2) is true. 
    // ----------------------------------------------------------------------
    __property dword Sequence[dword pArrayIndex] = { read=FGetSequence, 
    write=FSetSequence }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of a single 
    //              internal DIDEVICEOBJECTDATA structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    SizeAll
    // Description: The SizeAll property contains the size in bytes of all 
    //              internal DIDEVICEOBJECTDATA structures. This result is 
    //              the same as multiplying TDIDeviceObjectData::ArraySize 
    //              and TDIDeviceObjectData::Size. 
    // ----------------------------------------------------------------------
    __property dword SizeAll = { read=FGetSizeAll, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    TimeStamp
    // Description: The TimeStamp property defines the tick count, in 
    //              milliseconds, when the call was made. 
    //              
    //              When calling TDx_InputDevice::SendDeviceData(), this 
    //              property must be 0. 
    //              
    //              A lower resolution system tick count value can be 
    //              obtained with the Win32 GetTickCount function. 
    //              The tick count wraps around approximately every 50 days. 
    // ----------------------------------------------------------------------
    __property dword TimeStamp[dword pArrayIndex] = { read=FGetTimeStamp, 
    write=FSetTimeStamp }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDIDeviceObjectData's internal DIDEVICEOBJECTDATA 
    //              structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // Params:      pArrayIndex - 
    //                The ArrayIndex parameter determines which internal 
    //                structure needs to be cleared. Correct values range from 
    //                0 to TDIDeviceObjectData::ArraySize-1. 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear( dword pArrayIndex ); 

    // ----------------------------------------------------------------------
    // Method:      ClearAll()
    // Description: The ClearAll() method can be used to clear the contents 
    //              of all the internal DIDEVICEOBJECTDATA structures of the 
    //              TDIDeviceObjectData component. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall ClearAll(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDIDeviceObjectData(TComponent* Owner); 
    virtual __fastcall ~TDIDeviceObjectData(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DIDEVICEOBJECTDATA* Internal_DIDEVICEOBJECTDATA_Ptr = { 
    read=FGetInternal_DIDEVICEOBJECTDATA_Ptr, nodefault }; 

    void __fastcall Internal_DIDEVICEOBJECTDATA_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetArraySize(); 
    void __fastcall FSetArraySize( dword pArraySize ); 
    dword __fastcall FGetData( dword pArrayIndex ); 
    void __fastcall FSetData( dword pArrayIndex, dword pData ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetOfs( dword pArrayIndex ); 
    void __fastcall FSetOfs( dword pArrayIndex, dword pOfs ); 
    dword __fastcall FGetSequence( dword pArrayIndex ); 
    void __fastcall FSetSequence( dword pArrayIndex, dword pSequence ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    dword __fastcall FGetSizeAll(); 
    dword __fastcall FGetTimeStamp( dword pArrayIndex ); 
    void __fastcall FSetTimeStamp( dword pArrayIndex, dword pTimeStamp ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DIDEVICEOBJECTDATA* __fastcall FGetInternal_DIDEVICEOBJECTDATA_Ptr(); 
    void __fastcall FSetInternal_DIDEVICEOBJECTDATA( dword pArrayIndex, 
    DIDEVICEOBJECTDATA* pDIDEVICEOBJECTDATA ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    dword fArraySize; 
    HRESULT fErrorValue; 
    dword fSize; 
    dword fSizeAll; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DIDEVICEOBJECTDATA* fDIDEVICEOBJECTDATA; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
