#ifndef TDIDeviceObjectInstanceH
#define TDIDeviceObjectInstanceH

// ==========================================================================
// File:         TDIDeviceObjectInstance.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons, website),
//               Brian Austin (some source code, documentation)
// Description:  This file defines the TDIDeviceObjectInstance Component 
//
// "TDx_Input_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Input_Library_Defns.H"
#include "TDx_Input_Library_Functions.H"

// ==========================================================================
// Set:         TDIDeviceObjectInstanceFlags_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TDIDeviceObjectInstance::Flags property using the BCB Object 
//              Inspector. For detailed descriptions of each flag, see the 
//              Flags property description below. 
// ==========================================================================
enum TDIDeviceObjectInstance_Flags_Flags
    {
    didoi_ffactuator,
    didoi_ffeffecttrigger,
    didoi_polled
    };
typedef Set< TDIDeviceObjectInstance_Flags_Flags, 
             didoi_ffactuator, 
             didoi_polled >  TDIDeviceObjectInstance_Flags_Set;
// ==========================================================================
// Set:         TDIDeviceObjectInstanceType_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TDIDeviceObjectInstance::Type property using the BCB Object 
//              Inspector. For detailed descriptions of each flag, see the 
//              Type property description below. 
// ==========================================================================
enum TDIDeviceObjectInstance_Type_Flags
    {
    didft_absaxis_,
    didft_all,
    didft_axis_,
    didft_button_,
    didft_collection,
    didft_ffactuator_,
    didft_ffeffecttrigger_,
    didft_nocollection,
    didft_nodata,
    didft_output,
    didft_pov_,
    didft_pshbutton_,
    didft_relaxis_,
    didft_tglbutton_,
    didft_vendordefined_
    };
typedef Set< TDIDeviceObjectInstance_Type_Flags, 
             didft_absaxis_, 
             didft_vendordefined_ >  TDIDeviceObjectInstance_Type_Set;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDIDeviceObjectInstance
// Description: The TDIDeviceObjectInstance component wraps the DirectX 
//              DIDEVICEOBJECTINSTANCE structure containing the information 
//              about a device object instance. 
//              
//              This component is used in the 
//              TDx_InputDevice::OnEnumObjects() event and the 
//              TDx_InputDevice::GetObjectInfo method to provide information 
//              about a particular object associated with a device. 
//              
//              Each TDIDeviceObjectInstance component contains the details 
//              of an individual device control such as a button or a single 
//              joystick or mouse axis. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDIDeviceObjectInstance : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDIDeviceObjectInstance : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Flags_
    // Description: The Flags property defines flags indicating additional 
    //              attributes of the device object instance. 
    //              The described effect applies when the flag is set. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'Flags' property via the BCB Object Inspector. 
    //               For detailed descriptions of each flag, see the Flags 
    //              property description below. 
    // ----------------------------------------------------------------------
    __property TDIDeviceObjectInstance_Flags_Set Flags_ = { read=FGetFlags_, 
    write=FSetFlags_, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Type_
    // Description: The Type property is unusual, in that it defines flags 
    //              indicating the object type and it also identifies the 
    //              object instance number in the middle 16 bits. 
    //              
    //              The object instance number can be extracted using the 
    //              DIDFT_GETINSTANCE macro: 
    //              
    //                DIDFT_GETINSTANCE(n)   LOWORD((n) >> 8) 
    //              
    //              where n is one of the TDIDeviceObjectInstance::Type flag 
    //              values. 
    //              
    //              The described effect applies when the flag is set. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'Type' property via the BCB Object Inspector. 
    //               For detailed descriptions of each flag, see the Type 
    //              property description below. 
    // ----------------------------------------------------------------------
    __property TDIDeviceObjectInstance_Type_Set Type_ = { read=FGetType_, 
    write=FSetType_, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    CollectionNumber
    // Description: The CollectionNumber property defines which HID link 
    //              collection this object belongs to. 
    // ----------------------------------------------------------------------
    __property word CollectionNumber = { read=FGetCollectionNumber, 
    write=FSetCollectionNumber, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    DesignatorIndex
    // Description: The DesignatorIndex defines an index to a designator in 
    //              the HID physical descriptor. 
    //              
    //              Additional information about the device object can be 
    //              retrieved by passing this number to functions in the HID 
    //              parsing library (Hidpi.h). 
    // ----------------------------------------------------------------------
    __property word DesignatorIndex = { read=FGetDesignatorIndex, 
    write=FSetDesignatorIndex, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Dimension
    // Description: The Dimension property defines the units used when 
    //              reporting a value for the object. 
    //              
    //              This property can be used to distinguish between the 
    //              position and velocity of a control but will be zero if 
    //              the dimensional units are not known. 
    // ----------------------------------------------------------------------
    __property dword Dimension = { read=FGetDimension, write=FSetDimension, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Exponent
    // Description: The Exponent property defines the exponent associated 
    //              with the dimension, if known. 
    //              
    //              Dimensional units are always integral, this exponent may 
    //              be required for conversion to non-integral types. 
    // ----------------------------------------------------------------------
    __property word Exponent = { read=FGetExponent, write=FSetExponent, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    FFForceResolution
    // Description: The FFForceResolution defines how many different force 
    //              output levels the actuator associated with this object is 
    //              capable of outputting. 
    //              
    //              A scale of 0 to 10,000 units represents the range of 
    //              possible force values, from none at 0, up to maximum 
    //              output at 10,000. 
    //              
    //              For example, if this property is 20, there are 20 
    //              possible levels of force output and thus changes in the 
    //              actual force output will occur at each 500 unit increment 
    //              in force value. 
    // ----------------------------------------------------------------------
    __property dword FFForceResolution = { read=FGetFFForceResolution, 
    write=FSetFFForceResolution, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    FFMaxForce
    // Description: The FFMaxForce property defines the maximum force, in 
    //              newtons, the actuator associated with this object is 
    //              capable of creating. 
    // ----------------------------------------------------------------------
    __property dword FFMaxForce = { read=FGetFFMaxForce, 
    write=FSetFFMaxForce, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Name
    // Description: The Name property defines the friendly name of the 
    //              object. 
    //              
    //              A friendly name will be something like 'Button 3' or 
    //              'Space Bar'. 
    // ----------------------------------------------------------------------
    __property AnsiString Name = { read=FGetName, write=FSetName, nodefault 
    }; 

    // ----------------------------------------------------------------------
    // Property:    Ofs
    // Description: The Ofs property defines the offset within the data 
    //              format at which data is reported for this object. 
    //              
    //              This value can be used to identify the object in method 
    //              calls and objects accepting the DIPH_BYOFFSET flag. 
    // ----------------------------------------------------------------------
    __property dword Ofs = { read=FGetOfs, write=FSetOfs, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    ReportId
    // Description: The ReportId property is reserved and should be set to 0. 
    // ----------------------------------------------------------------------
    __property word ReportId = { read=FGetReportId, write=FSetReportId, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property defines the size, in bytes, of the 
    //              internal structure used by this component. 
    //              
    //              This property may be inspected during enumeration to 
    //              determine how many properties of the component are valid. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, 
    default=sizeof(DIDEVICEOBJECTINSTANCE) }; 

    // ----------------------------------------------------------------------
    // Property:    Usage
    // Description: The Usage property defines the HID usage code. 
    //              
    //              Usage codes are optional for non-HID devices. 
    // ----------------------------------------------------------------------
    __property word Usage = { read=FGetUsage, write=FSetUsage, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    UsagePage
    // Description: The UsagePage property defines the HID usage page code. 
    //              
    //              Usage page codes are optional for non-HID devices. 
    // ----------------------------------------------------------------------
    __property word UsagePage = { read=FGetUsagePage, write=FSetUsagePage, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDIDeviceObjectInstance component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDIDeviceObjectInstance method 
    //              or fget/fset. eg. DI_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DI_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Flags
    // Description: The Flags property defines flags indicating additional 
    //              attributes of the device object instance. 
    //              The described effect applies when the flag is set. 
    // Flags:       DIDOI_ASPECTACCEL -
    //                This device object instance can report acceleration 
    //                information. 
    //                
    //              DIDOI_ASPECTFORCE -
    //                This device object instance can report force 
    //                information. 
    //              DIDOI_ASPECTMASK -
    //                This flag contains the bits that are used to report 
    //                aspect information. 
    //                
    //                An object can only represent one aspect. 
    //              DIDOI_ASPECTPOSITION -
    //                This device object instance can report position 
    //                information. 
    //              DIDOI_ASPECTVELOCITY -
    //                This device object instance can report velocity 
    //                information. 
    //              DIDOI_FFACTUATOR -
    //                This device object instance is capable of applying 
    //                force feedback effects. 
    //              DIDOI_FFEFFECTTRIGGER -
    //                The device object instance can trigger the playback of 
    //                force feedback effects, which is useful when using 
    //                condition effects. 
    //              DIDOI_POLLED -
    //                The device object instance must be polled to obtain 
    //                data. 
    //                
    //                TDx_InputDevice::Poll() must be explicitly called in 
    //                order to obtain data from the object. 
    // ----------------------------------------------------------------------
    __property dword Flags = { read=FGetFlags, write=FSetFlags, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    GuidType
    // Description: The GuidType property defines a device type that 
    //              describes this device object instance. 
    //              
    //              This property is optional and the possible values are: 
    //              
    //              GUID_XAxis       The horizontal axis. 
    //              GUID_YAxis       The vertical axis. 
    //              GUID_ZAxis       The z-axis. 
    //              GUID_RxAxis     Rotation about the x-axis. 
    //              GUID_RyAxis     Rotation about the y-axis. 
    //              GUID_RzAxis     Rotation about the z-axis 
    //              GUID_Slider       A slider axis. 
    //              GUID_Button      A mouse button. 
    //              GUID_Key          A keyboard key. 
    //              GUID_POV         A point-of-view indicator or "hat". 
    //              GUID_Unknown Unknown object. 
    // ----------------------------------------------------------------------
    __property GUID GuidType = { read=FGetGuidType, write=FSetGuidType, 
    nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Type
    // Description: The Type property is unusual, in that it defines flags 
    //              indicating the object type and it also identifies the 
    //              object instance number in the middle 16 bits. 
    //              
    //              The object instance number can be extracted using the 
    //              DIDFT_GETINSTANCE macro: 
    //              
    //                DIDFT_GETINSTANCE(n)   LOWORD((n) >> 8) 
    //              
    //              where n is one of the TDIDeviceObjectInstance::Type flag 
    //              values. 
    //              
    //              The described effect applies when the flag is set. 
    // Flags:       DIDFT_ABSAXIS -
    //                Enumerate any object with an absolute axis. 
    //              DIDFT_ALL -
    //                Enumerate all objects. 
    //              DIDFT_AXIS -
    //                Enumerate any object with an absolute or relative axis. 
    //              DIDFT_BUTTON -
    //                Enumerate any object with push or toggle buttons. 
    //              DIDFT_COLLECTION -
    //                Enumerate HID link collections. 
    //              DIDFT_ENUMCOLLECTION(n) -
    //                Enumerate objects that belong to HID link collection 
    //                number n. 
    //              DIDFT_FFACTUATOR -
    //                Enumerate objects to which force-feedback may be 
    //                applied. 
    //              DIDFT_FFEFFECTTRIGGER -
    //                Enumerate objects that can trigger force-feedback 
    //                effects. 
    //              DIDFT_NOCOLLECTION -
    //                Enumerate objects that do not belong to any HID link 
    //                collection. 
    //                Objects falling into this category are those that have 
    //                TDIDeviceObjectInstance::CollectionNumber = 0. 
    //              DIDFT_NODATA -
    //                Enumerate objects that do not generate data. 
    //              DIDFT_OUTPUT -
    //                Enumerate objects to which data can be sent using 
    //                TDx_InputDevice::SendDeviceData(). 
    //              DIDFT_POV -
    //                Enumerate point-of-view controllers. 
    //              DIDFT_PSHBUTTON -
    //                Enumerate push buttons. 
    //                Push buttons is reported as down when pressed, and up 
    //                when released. 
    //              DIDFT_RELAXIS -
    //                Enumerate objects with a relative axis.. 
    //              DIDFT_TGLBUTTON -
    //                Enumerate objects that are toggle buttons. 
    //                Toggle buttons are reported down when pressed, and 
    //                remain down until re-pressed.. 
    //              DIDFT_VENDORDEFINED -
    //                Enumerate objects of a type defined by the device 
    //                manufacturer. 
    // ----------------------------------------------------------------------
    __property dword Type = { read=FGetType, write=FSetType, default=0 }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDIDeviceObjectInstance's internal 
    //              DIDEVICEOBJECTINSTANCE structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Property:    Flags_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the Flags property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* Flags_Strings = { read=FGetFlags_Strings };

    // ----------------------------------------------------------------------
    // Property:    Type_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the Type property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* Type_Strings = { read=FGetType_Strings };

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDIDeviceObjectInstance(TComponent* Owner); 
    virtual __fastcall ~TDIDeviceObjectInstance(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DIDEVICEOBJECTINSTANCE* Internal_DIDEVICEOBJECTINSTANCE_Ptr = 
    { read=FGetInternal_DIDEVICEOBJECTINSTANCE_Ptr, nodefault }; 

    void __fastcall Internal_DIDEVICEOBJECTINSTANCE_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Object Inspector 'Flag' Access Methods
    // ----------------------------------------------------------------------
    TDIDeviceObjectInstance_Flags_Set __fastcall FGetFlags_(); 
    void __fastcall FSetFlags_( TDIDeviceObjectInstance_Flags_Set pFlags_ ); 
    TDIDeviceObjectInstance_Type_Set __fastcall FGetType_(); 
    void __fastcall FSetType_( TDIDeviceObjectInstance_Type_Set pType_ ); 

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    word __fastcall FGetCollectionNumber(); 
    void __fastcall FSetCollectionNumber( word pCollectionNumber ); 
    word __fastcall FGetDesignatorIndex(); 
    void __fastcall FSetDesignatorIndex( word pDesignatorIndex ); 
    dword __fastcall FGetDimension(); 
    void __fastcall FSetDimension( dword pDimension ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    word __fastcall FGetExponent(); 
    void __fastcall FSetExponent( word pExponent ); 
    dword __fastcall FGetFFForceResolution(); 
    void __fastcall FSetFFForceResolution( dword pFFForceResolution ); 
    dword __fastcall FGetFFMaxForce(); 
    void __fastcall FSetFFMaxForce( dword pFFMaxForce ); 
    dword __fastcall FGetFlags(); 
    void __fastcall FSetFlags( dword pFlags ); 
    GUID __fastcall FGetGuidType(); 
    void __fastcall FSetGuidType( GUID pGuidType ); 
    AnsiString __fastcall FGetName(); 
    void __fastcall FSetName( AnsiString pName ); 
    dword __fastcall FGetOfs(); 
    void __fastcall FSetOfs( dword pOfs ); 
    word __fastcall FGetReportId(); 
    void __fastcall FSetReportId( word pReportId ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    dword __fastcall FGetType(); 
    void __fastcall FSetType( dword pType ); 
    word __fastcall FGetUsage(); 
    void __fastcall FSetUsage( word pUsage ); 
    word __fastcall FGetUsagePage(); 
    void __fastcall FSetUsagePage( word pUsagePage ); 

    // ----------------------------------------------------------------------
    // Flag_Strings Access Methods
    // ----------------------------------------------------------------------
    TStringList* __fastcall FGetFlags_Strings();

    TStringList* __fastcall FGetType_Strings();
// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DIDEVICEOBJECTINSTANCE* __fastcall 
    FGetInternal_DIDEVICEOBJECTINSTANCE_Ptr(); 
    void __fastcall FSetInternal_DIDEVICEOBJECTINSTANCE( 
    DIDEVICEOBJECTINSTANCE* pDIDEVICEOBJECTINSTANCE ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    GUID fGuidType; 

    // ----------------------------------------------------------------------
    // Flags_Strings Variables
    // ----------------------------------------------------------------------
    TStringList* fFlags_Strings;
    TStringList* fType_Strings;

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DIDEVICEOBJECTINSTANCE fDIDEVICEOBJECTINSTANCE; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
