#ifndef TDIEffEscapeH
#define TDIEffEscapeH

// ==========================================================================
// File:         TDIEffEscape.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons, website),
//               Brian Austin (some source code, documentation)
// Description:  This file defines the TDIEffEscape Component 
//
// "TDx_Input_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Input_Library_Defns.H"
#include "TDx_Input_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDIEffEscape
// Description: The TDIEffEscape component wraps the DirectX DIEEFFESCAPE 
//              structure which defines hardware specific data to be passed 
//              directly to the device driver. 
//              
//              This component is used by the TDx_InputDevice::Escape() and 
//              TDx_InputEffect::Escape() methods to pass hardware specific 
//              commands and information directly to the device driver. 
//              
//              TDIDeviceInstance::FFDriver should be checked against the 
//              expected driver identifier to ensure the escape is sent to 
//              the right driver. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDIEffEscape : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDIEffEscape : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Command
    // Description: The Command property defines which command is being sent 
    //              to the driver. 
    //              
    //              The driver documentation should list valid comand numbers 
    //              and their associated function. 
    // ----------------------------------------------------------------------
    __property dword Command = { read=FGetCommand, write=FSetCommand, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    InBufferSize
    // Description: The InBufferSize property defines the size, in bytes, of 
    //              the buffer referenced by TDIEffEscape::InBuffer. 
    // ----------------------------------------------------------------------
    __property dword InBufferSize = { read=FGetInBufferSize, 
    write=FSetInBufferSize, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    OutBufferSize
    // Description: The OutBufferSize property defines the size, in bytes, of 
    //              the buffer referenced by TDIEffEscape::OutBuffer when 
    //              calling the escape methods and indicates how many bytes 
    //              were actually written on returning. 
    //              
    //              The application should make sure the buffer is big enough 
    //              for the actual amount of output. 
    // ----------------------------------------------------------------------
    __property dword OutBufferSize = { read=FGetOutBufferSize, 
    write=FSetOutBufferSize, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property defines the size, in bytes, of the 
    //              internal structure used by this component. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, 
    default=sizeof(DIEFFESCAPE) }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDIEffEscape component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDIEffEscape method or 
    //              fget/fset. eg. DI_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DI_OK }; 

    // ----------------------------------------------------------------------
    // Property:    InBuffer
    // Description: The InBuffer property references a buffer with the data 
    //              required to perform the requested operation. 
    // ----------------------------------------------------------------------
    __property void* InBuffer = { read=FGetInBuffer, write=FSetInBuffer, 
    default=NULL }; 

    // ----------------------------------------------------------------------
    // Property:    OutBuffer
    // Description: The OutBuffer property references a buffer for holding 
    //              data returned by the requested operation. 
    // ----------------------------------------------------------------------
    __property void* OutBuffer = { read=FGetOutBuffer, write=FSetOutBuffer, 
    default=NULL }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDIEffEscape's internal DIEFFESCAPE structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDIEffEscape(TComponent* Owner); 
    virtual __fastcall ~TDIEffEscape(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DIEFFESCAPE* Internal_DIEFFESCAPE_Ptr = { 
    read=FGetInternal_DIEFFESCAPE_Ptr, nodefault }; 

    void __fastcall Internal_DIEFFESCAPE_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetCommand(); 
    void __fastcall FSetCommand( dword pCommand ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    void* __fastcall FGetInBuffer(); 
    void __fastcall FSetInBuffer( void* pInBuffer ); 
    dword __fastcall FGetInBufferSize(); 
    void __fastcall FSetInBufferSize( dword pInBufferSize ); 
    void* __fastcall FGetOutBuffer(); 
    void __fastcall FSetOutBuffer( void* pOutBuffer ); 
    dword __fastcall FGetOutBufferSize(); 
    void __fastcall FSetOutBufferSize( dword pOutBufferSize ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DIEFFESCAPE* __fastcall FGetInternal_DIEFFESCAPE_Ptr(); 
    void __fastcall FSetInternal_DIEFFESCAPE( DIEFFESCAPE* pDIEFFESCAPE ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DIEFFESCAPE fDIEFFESCAPE; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
