#ifndef TDIEffectInfoH
#define TDIEffectInfoH

// ==========================================================================
// File:         TDIEffectInfo.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons, website),
//               Brian Austin (some source code, documentation)
// Description:  This file defines the TDIEffectInfo Component 
//
// "TDx_Input_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Input_Library_Defns.H"
#include "TDx_Input_Library_Functions.H"

// ==========================================================================
// Set:         TDIEffectInfoDynamicParams_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TDIEffectInfo::DynamicParams property using the BCB Object 
//              Inspector. For detailed descriptions of each flag, see the 
//              DynamicParams property description below. 
// ==========================================================================
enum TDIEffectInfo_DynamicParams_Flags
    {
    diep_axes,
    diep_direction,
    diep_duration,
    diep_envelope,
    diep_gain,
    diep_sampleperiod,
    diep_startdelay,
    diep_triggerbutton,
    diep_triggerrepeatinterval,
    diep_typespecificparams
    };
typedef Set< TDIEffectInfo_DynamicParams_Flags, 
             diep_axes, 
             diep_typespecificparams >  TDIEffectInfo_DynamicParams_Set;
// ==========================================================================
// Set:         TDIEffectInfoEffectType_Set
// Description: A set of mangled DirectX flag names that are used to set the 
//              TDIEffectInfo::EffectType property using the BCB Object 
//              Inspector. For detailed descriptions of each flag, see the 
//              EffectType property description below. 
// ==========================================================================
enum TDIEffectInfo_EffectType_Flags
    {
    dieft_deadband,
    dieft_ffattack,
    dieft_fffade,
    dieft_periodic,
    dieft_posnegcoefficients,
    dieft_posnegsaturation,
    dieft_saturation
    };
typedef Set< TDIEffectInfo_EffectType_Flags, 
             dieft_deadband, 
             dieft_saturation >  TDIEffectInfo_EffectType_Set;

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDIEffectInfo
// Description: The TDIEffectInfo component wraps the DirectX DIEFFECTINFO 
//              structure containing information on the type and capabilities 
//              of an effect. 
//              
//              This component is used by the TDx_InputDevice::EnumEffects() 
//              and TDx_InputDevice::GetEffectInfo() methods to return 
//              information about a particular effect supported by a device. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDIEffectInfo : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDIEffectInfo : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    DynamicParams_
    // Description: The DynamicParams property defines flags indicating which 
    //              effect parameters can be modified without halting 
    //              playback. 
    //              The properties noted in each flags description may be 
    //              modified dynamically when that flag is set. 
    //              
    //              If an attempt is made to change a property indicated by 
    //              these flags while the effect is playing and dynamic 
    //              modification is not supported by the driver, then the 
    //              effect will be stopped, updated and restarted. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'DynamicParams' property via the BCB Object 
    //              Inspector.  For detailed descriptions of each flag, see 
    //              the DynamicParams property description below. 
    // ----------------------------------------------------------------------
    __property TDIEffectInfo_DynamicParams_Set DynamicParams_ = { 
    read=FGetDynamicParams_, write=FSetDynamicParams_, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    EffectType_
    // Description: The EffectType property defines flags indicating the type 
    //              and capabilities of an effect or define what effects are 
    //              to be enumerated. 
    //              The described effect applies when the flag is set. 
    //              
    //              Type and capability information indicated by the presence 
    //              of the flag are described first, then any effect the flag 
    //              has on TDx_InputDevice::EnumEffects() is described. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'EffectType' property via the BCB Object 
    //              Inspector.  For detailed descriptions of each flag, see 
    //              the EffectType property description below. 
    // ----------------------------------------------------------------------
    __property TDIEffectInfo_EffectType_Set EffectType_ = { 
    read=FGetEffectType_, write=FSetEffectType_, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    StaticParams_
    // Description: The StaticParams property defines flags indicating which 
    //              parameters of an effect are supported. 
    //              The properties noted in each flags description are 
    //              supported when that flag is set. 
    //              
    //              Attempts to use unsupported parameters will not cause 
    //              errors, unsupported properties are merely ignored. 
    //              These flags can be used to tailor the use of force 
    //              feedback to the capabilities of a specific device. 
    // Note:        This property is used to set the various DirectX Flags 
    //              for the 'StaticParams' property via the BCB Object 
    //              Inspector.  For detailed descriptions of each flag, see 
    //              the StaticParams property description below. 
    // ----------------------------------------------------------------------
    __property TDIEffectInfo_DynamicParams_Set StaticParams_ = { 
    read=FGetStaticParams_, write=FSetStaticParams_, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    Name
    // Description: The Name property defines a friendly name for the effect 
    //              such as 'Sawtooth up' or 'Constant force'. 
    // ----------------------------------------------------------------------
    __property AnsiString Name = { read=FGetName, write=FSetName, nodefault 
    }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property defines the size, in bytes, of the 
    //              internal structure used by this component. 
    //              
    //              This property may be inspected during enumeration to 
    //              determine how many properties of the component are valid. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, 
    default=sizeof(DIEFFECTINFO) }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDIEffectInfo component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    DynamicParams
    // Description: The DynamicParams property defines flags indicating which 
    //              effect parameters can be modified without halting 
    //              playback. 
    //              The properties noted in each flags description may be 
    //              modified dynamically when that flag is set. 
    //              
    //              If an attempt is made to change a property indicated by 
    //              these flags while the effect is playing and dynamic 
    //              modification is not supported by the driver, then the 
    //              effect will be stopped, updated and restarted. 
    // Flags:       DIEP_AXES -
    //                The TDIEffect::Axes and TDIEffect::AxesCount 
    //                properties. 
    //              DIEP_DIRECTION -
    //                The TDIEffect::Direction and TDIEffect::AxesCount 
    //                properties. 
    //              DIEP_DURATION -
    //                The TDIEffect::Duration property. 
    //              DIEP_ENVELOPE -
    //                The TDIEffect::Envelope property. 
    //              DIEP_GAIN -
    //                The TDIEffect::Gain property. 
    //              DIEP_SAMPLEPERIOD -
    //                The TDIEffect::SamplePeriod property. 
    //              DIEP_STARTDELAY -
    //                The TDIEffect::StartDelay property. 
    //              DIEP_TRIGGERBUTTON -
    //                The TDIEffect::TriggerButton property. 
    //              DIEP_TRIGGERREPEATINTERVAL -
    //                The TDIEffect::TriggerRepeatInterval property. 
    //              DIEP_TYPESPECIFICPARAMS -
    //                The TDIEffect::TypeSpecificParams property. 
    // ----------------------------------------------------------------------
    __property dword DynamicParams = { read=FGetDynamicParams, 
    write=FSetDynamicParams, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    EffectType
    // Description: The EffectType property defines flags indicating the type 
    //              and capabilities of an effect or define what effects are 
    //              to be enumerated. 
    //              The described effect applies when the flag is set. 
    //              
    //              Type and capability information indicated by the presence 
    //              of the flag are described first, then any effect the flag 
    //              has on TDx_InputDevice::EnumEffects() is described. 
    // Flags:       DIEFT_ALL -
    //                All effects should be enumerated, regardless of type. 
    //                
    //                This flag is only usable with 
    //                TDx_InputDevice::EnumEffects() and no other flag may be 
    //                used when this one is set. 
    //              DIEFT_CONDITION -
    //                The effect represents a condition or only conditions 
    //                are to be enumerated. 
    //                
    //                When you are creating or modifying a condition, the 
    //                TDIEffect::TypeSpecificParams array must contain one 
    //                TDICondition component for each axis affected and 
    //                TDIEffect::TypeSpecificParamsSize must be set to a 
    //                value equal to TDIEffect::AxesCount multiplied by 
    //                sizeof(TDICondition). 
    //              DIEFT_CONSTANTFORCE -
    //                The effect represents a constant force effect or only 
    //                constant force effects are to be enumerated. 
    //                
    //                When you are creating or modifying a constant force, 
    //                the TDIEffect::TypeSpecificParams array must contain 
    //                one TDIConstantForce component and 
    //                TDIEffect::TypeSpecificParamsSize must be set to 
    //                sizeof(TDIConstanForce). 
    //              DIEFT_CUSTOMFORCE -
    //                The effect represents a custom force effect or only 
    //                custom force effects are to be enumerated. 
    //                
    //                When you are creating or modifying a custom force, the 
    //                TDIEffect::TypeSpecificParams array must contain one 
    //                TDICustomForce component and 
    //                TDIEffect::TypeSpecificParamsSize must be set to 
    //                sizeof(TDICustomForce). 
    //              DIEFT_DEADBAND -
    //                Support for TDICondition::DeadBand is available. 
    //              DIEFT_FFATTACK -
    //                Support for TDIEnvelope::AttackLevel and 
    //                TDIEnvelope::AttackTime is available for this effect.. 
    //                
    //                When both DIEFT_FFATTACK and DIEFT_FFFADE are both not 
    //                set, then envelopes will be ignored for this effect. 
    //              DIEFT_FFFADE -
    //                Support for TDIEnvelope::FadeLevel and 
    //                TDIEnvelope::FadeTime is available for this effect.. 
    //                
    //                When both DIEFT_FFATTACK and DIEFT_FFFADE are both not 
    //                set, then envelopes will be ignored for this effect. 
    //              DIEFT_HARDWARE -
    //                The effect represents a hardware specific effect or 
    //                only hardware specific effects are to be enumerated. 
    //                
    //                The documentation provided by the device manufacturer 
    //                should provide further information about hardware 
    //                specific effects. 
    //              DIEFT_PERIODIC -
    //                The effect represents a periodic effect or only 
    //                periodic effects are to be enumerated. 
    //                
    //                When you are creating or modifying a periodic force, 
    //                the TDIEffect::TypeSpecificParams array must contain 
    //                one TDIPeriodic component and 
    //                TDIEffect::TypeSpecificParamsSize must be set to 
    //                sizeof(TDIPeriodic). 
    //              DIEFT_POSNEGCOEFFICIENTS -
    //                Support for both TDICondition::PositiveCoefficient and 
    //                TDICondition::NegativeCoefficient is available. 
    //                
    //                If this flag is not set, 
    //                TDICondition::PositiveCoefficient will be used to 
    //                represent both coefficients. 
    //              DIEFT_POSNEGSATURATION -
    //                Support for both TDICondition::PositiveSaturation and 
    //                TDICondition::NegativeSaturation is available. 
    //                
    //                If this flag is not set, 
    //                TDICondition::PositiveSaturation will be used to 
    //                represent both saturation values. 
    //              DIEFT_RAMPFORCE -
    //                The effect represents a ramp force effect or only ramp 
    //                force effects are to be enumerated. 
    //                
    //                When you are creating or modifying a ramp force, the 
    //                TDIEffect::TypeSpecificParams array must contain one 
    //                TDIRampForce component and 
    //                TDIEffect::TypeSpecificParamsSize must be set to 
    //                sizeof(TDIRampForce). 
    //              DIEFT_SATURATION -
    //                Support for saturation of condition effects is 
    //                available. 
    //                
    //                When saturation is not supported, both 
    //                TDICondition::PositiveSaturation and 
    //                TDICondition::NegativeSaturation will be ignored and 
    //                the force generated is only limited by the maximum the 
    //                device can output. 
    // ----------------------------------------------------------------------
    __property dword EffectType = { read=FGetEffectType, 
    write=FSetEffectType, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDIEffectInfo method or 
    //              fget/fset. eg. DI_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DI_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Guid
    // Description: The Guid property defines a GUID identifying the effect. 
    //              
    //              Possible values include: 
    //              
    //              GUID_ConstantForce 
    //              GUID_RampForce 
    //              GUID_Square 
    //              GUID_Sine 
    //              GUID_Triangle 
    //              GUID_SawtoothUp 
    //              GUID_SawtoothDown 
    //              GUID_Spring 
    //              GUID_Damper 
    //              GUID_Inertia 
    //              GUID_Friction 
    //              GUID_CustomForce 
    // ----------------------------------------------------------------------
    __property GUID Guid = { read=FGetGuid, write=FSetGuid, nodefault }; 

    // ----------------------------------------------------------------------
    // Property:    StaticParams
    // Description: The StaticParams property defines flags indicating which 
    //              parameters of an effect are supported. 
    //              The properties noted in each flags description are 
    //              supported when that flag is set. 
    //              
    //              Attempts to use unsupported parameters will not cause 
    //              errors, unsupported properties are merely ignored. 
    //              These flags can be used to tailor the use of force 
    //              feedback to the capabilities of a specific device. 
    // ----------------------------------------------------------------------
    __property dword StaticParams = { read=FGetStaticParams, 
    write=FSetStaticParams, default=0 }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDIEffectInfo's internal DIEFFECTINFO structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Property:    DynamicParams_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the DynamicParams property. Note: Any 
    //              memory associated with these strings is released on 
    //              destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* DynamicParams_Strings = { read=FGetDynamicParams_Strings };

    // ----------------------------------------------------------------------
    // Property:    EffectType_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the EffectType property. Note: Any memory 
    //              associated with these strings is released on destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* EffectType_Strings = { read=FGetEffectType_Strings };

    // ----------------------------------------------------------------------
    // Property:    StaticParams_Strings()
    // Description: This property returns a TStringList containing current 
    //              flag values for the StaticParams property. Note: Any 
    //              memory associated with these strings is released on 
    //              destruction. 
    // ----------------------------------------------------------------------
    __property TStringList* StaticParams_Strings = { read=FGetStaticParams_Strings };

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDIEffectInfo(TComponent* Owner); 
    virtual __fastcall ~TDIEffectInfo(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DIEFFECTINFO* Internal_DIEFFECTINFO_Ptr = { 
    read=FGetInternal_DIEFFECTINFO_Ptr, nodefault }; 

    void __fastcall Internal_DIEFFECTINFO_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Object Inspector 'Flag' Access Methods
    // ----------------------------------------------------------------------
    TDIEffectInfo_DynamicParams_Set __fastcall FGetDynamicParams_(); 
    void __fastcall FSetDynamicParams_( TDIEffectInfo_DynamicParams_Set 
    pDynamicParams_ ); 
    TDIEffectInfo_EffectType_Set __fastcall FGetEffectType_(); 
    void __fastcall FSetEffectType_( TDIEffectInfo_EffectType_Set 
    pEffectType_ ); 
    TDIEffectInfo_DynamicParams_Set __fastcall FGetStaticParams_(); 
    void __fastcall FSetStaticParams_( TDIEffectInfo_DynamicParams_Set 
    pStaticParams_ ); 

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetDynamicParams(); 
    void __fastcall FSetDynamicParams( dword pDynamicParams ); 
    dword __fastcall FGetEffectType(); 
    void __fastcall FSetEffectType( dword pEffectType ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    GUID __fastcall FGetGuid(); 
    void __fastcall FSetGuid( GUID pGuid ); 
    AnsiString __fastcall FGetName(); 
    void __fastcall FSetName( AnsiString pName ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    dword __fastcall FGetStaticParams(); 
    void __fastcall FSetStaticParams( dword pStaticParams ); 

    // ----------------------------------------------------------------------
    // Flag_Strings Access Methods
    // ----------------------------------------------------------------------
    TStringList* __fastcall FGetDynamicParams_Strings();

    TStringList* __fastcall FGetEffectType_Strings();
    TStringList* __fastcall FGetStaticParams_Strings();
// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DIEFFECTINFO* __fastcall FGetInternal_DIEFFECTINFO_Ptr(); 
    void __fastcall FSetInternal_DIEFFECTINFO( DIEFFECTINFO* pDIEFFECTINFO ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    GUID fGuid; 

    // ----------------------------------------------------------------------
    // Flags_Strings Variables
    // ----------------------------------------------------------------------
    TStringList* fDynamicParams_Strings;
    TStringList* fEffectType_Strings;
    TStringList* fStaticParams_Strings;

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DIEFFECTINFO fDIEFFECTINFO; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
