#ifndef TDIEnvelopeH
#define TDIEnvelopeH

// ==========================================================================
// File:         TDIEnvelope.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons, website),
//               Brian Austin (some source code, documentation)
// Description:  This file defines the TDIEnvelope Component 
//
// "TDx_Input_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Input_Library_Defns.H"
#include "TDx_Input_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDIEnvelope
// Description: The TDIEnvelope component wraps the DirectX DIENVELOPE 
//              structure, which contains information about an envelope which 
//              can optionally be used to modify the shape of constant force 
//              and periodic effects. 
//              
//              This component is used as part of the TDIEffect component to 
//              specify optional envelope parameters used with an effect. 
//              
//              TDIPeriodic::Magnitude or TDIConstantForce::Magnitude defines 
//              the sustain level, while TDIEffect::Duration defines the 
//              sustain time. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDIEnvelope : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDIEnvelope : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    AttackLevel
    // Description: The AttackLevel property defines the amplitude used at 
    //              the start of the envelope. 
    //              
    //              This value ranges from 0 to 10,000, relative to 0 for 
    //              constant forces, or relative to TDIPeriodic::Offset for 
    //              periodic forces. 
    // ----------------------------------------------------------------------
    __property dword AttackLevel = { read=FGetAttackLevel, 
    write=FSetAttackLevel, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    AttackTime
    // Description: The AttackTime property defines the time, in 
    //              microseconds, it takes for the effect to reach the 
    //              sustain level. 
    // ----------------------------------------------------------------------
    __property dword AttackTime = { read=FGetAttackTime, 
    write=FSetAttackTime, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    FadeLevel
    // Description: The FadeLevel property defines the amplitude used at the 
    //              end of the envelope. 
    //              
    //              This value ranges from 0 to 10,000 relative to 0 for 
    //              constant forces or to TDIPeriodic::Offset for periodic 
    //              forces. 
    // ----------------------------------------------------------------------
    __property dword FadeLevel = { read=FGetFadeLevel, write=FSetFadeLevel, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    FadeTime
    // Description: The FadeTime property defines the time, in microseconds, 
    //              it takes for the effect to reach the fade level. 
    // ----------------------------------------------------------------------
    __property dword FadeTime = { read=FGetFadeTime, write=FSetFadeTime, 
    default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property defines the size, in bytes, of the 
    //              internal structure used by this component. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, 
    default=sizeof(DIENVELOPE) }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDIEnvelope component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDIEnvelope method or 
    //              fget/fset. eg. DI_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DI_OK }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDIEnvelope's internal DIENVELOPE structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDIEnvelope(TComponent* Owner); 
    virtual __fastcall ~TDIEnvelope(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DIENVELOPE* Internal_DIENVELOPE_Ptr = { 
    read=FGetInternal_DIENVELOPE_Ptr, nodefault }; 

    void __fastcall Internal_DIENVELOPE_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    dword __fastcall FGetAttackLevel(); 
    void __fastcall FSetAttackLevel( dword pAttackLevel ); 
    dword __fastcall FGetAttackTime(); 
    void __fastcall FSetAttackTime( dword pAttackTime ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetFadeLevel(); 
    void __fastcall FSetFadeLevel( dword pFadeLevel ); 
    dword __fastcall FGetFadeTime(); 
    void __fastcall FSetFadeTime( dword pFadeTime ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DIENVELOPE* __fastcall FGetInternal_DIENVELOPE_Ptr(); 
    void __fastcall FSetInternal_DIENVELOPE( DIENVELOPE* pDIENVELOPE ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DIENVELOPE fDIENVELOPE; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
