#ifndef TDIJoyStateH
#define TDIJoyStateH

// ==========================================================================
// File:         TDIJoyState.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons, website),
//               Brian Austin (some source code, documentation)
// Description:  This file defines the TDIJoyState Component 
//
// "TDx_Input_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Input_Library_Defns.H"
#include "TDx_Input_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDIJoyState
// Description: The TDIJoyState component wraps the DirectX DIJOYSTATE 
//              structure which holds information about the state of a basic 
//              joystick device. 
//              
//              This component is used with the 
//              TDx_InputDevice::GetDeviceState() method to obtain current 
//              device information, and with the 
//              TDx_InputDevice::SetDataFormat() method to set the devices 
//              data format. 
//              
//              When axes are in relative mode, the appropriate properties 
//              will describe the change since last reported. 
//              When axes are in absolute mode, the appropriate properties 
//              will describe the position relative to a fixed point. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDIJoyState : public TDIData
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDIJoyState : public TDIData
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    POV
    // Description: The POV property defines a dword array indicating the 
    //              position, in 100ths of a degree, of up to 4 direction 
    //              controllers such as a Point Of View hat. 
    //              
    //              Direction is measured clockwise from directly away from 
    //              the user and the centered position is reported as -1 or 
    //              65535. 
    //              
    //              Thus, for an indicator with five positions, it could have 
    //              a value of -1(or 65535) (centered), 0 (away from user), 
    //              9,000 (to the right), 18,000 (towards the user), or 
    //              27,000 (to the left). 
    // ----------------------------------------------------------------------
    __property dword POV[dword pIndex] = { read=FGetPOV, write=FSetPOV }; 

    // ----------------------------------------------------------------------
    // Property:    RX
    // Description: The RX property describes rotation of the joysticks X 
    //              axis. 
    //              
    //              When rotation is unsupported, this property will be 0. 
    // ----------------------------------------------------------------------
    __property long RX = { read=FGetRX, write=FSetRX, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    RY
    // Description: The RY property describes rotation of the joysticks Y 
    //              axis. 
    //              
    //              When rotation is unsupported, this property will be 0. 
    // ----------------------------------------------------------------------
    __property long RY = { read=FGetRY, write=FSetRY, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    RZ
    // Description: The RZ property describes rotation of the joysticks Z 
    //              axis and is often called the rudder. 
    //              
    //              When rotation is unsupported or the Z axis does not 
    //              exist, this property will be 0. 
    // ----------------------------------------------------------------------
    __property long RZ = { read=FGetRZ, write=FSetRZ, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Slider
    // Description: The Slider property defines an long array describing two 
    //              additional axis values whose semantics depend on the 
    //              specific joystick being used. 
    //              
    //              TDx_InputDevice::GetObjectInfo() can be used to obtain 
    //              semantic information about these axes, which were 
    //              formally called the U and V axes. 
    // ----------------------------------------------------------------------
    __property long Slider[dword pIndex] = { read=FGetSlider, 
    write=FSetSlider }; 

    // ----------------------------------------------------------------------
    // Property:    X
    // Description: The X property describes movement of the joysticks X 
    //              axis, which usually represents the left right movement of 
    //              a control stick. 
    // ----------------------------------------------------------------------
    __property long X = { read=FGetX, write=FSetX, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Y
    // Description: The Y property describes movement of the joysticks Y 
    //              axis, which usually represents the forward backward 
    //              movement of a control stick. 
    // ----------------------------------------------------------------------
    __property long Y = { read=FGetY, write=FSetY, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Z
    // Description: The Z property describes movement of the joysticks Z 
    //              axis, which usually represents a throttle control. 
    //              
    //              When there is no Z axis, this property will be 0. 
    // ----------------------------------------------------------------------
    __property long Z = { read=FGetZ, write=FSetZ, default=0 }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDIJoyState component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Buttons
    // Description: The Buttons property defines a byte array of up to 32 
    //              button states. 
    //              
    //              When a given button is down, the high-order bit of the 
    //              corresponding byte will be set. 
    //              If the button is up, or does not exist, the bit will be 
    //              clear. 
    // ----------------------------------------------------------------------
    __property byte Buttons[dword pIndex] = { read=FGetButtons, 
    write=FSetButtons }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDIJoyState method or 
    //              fget/fset. eg. DI_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DI_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of the 
    //              internal DIJOYSTATE structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDIJoyState's internal DIJOYSTATE structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDIJoyState(TComponent* Owner); 
    virtual __fastcall ~TDIJoyState(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DIJOYSTATE* Internal_DIJOYSTATE_Ptr = { 
    read=FGetInternal_DIJOYSTATE_Ptr, nodefault }; 

    void __fastcall Internal_DIJOYSTATE_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    byte __fastcall FGetButtons( dword pIndex ); 
    void __fastcall FSetButtons( dword pIndex, byte pButtons ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetPOV( dword pIndex ); 
    void __fastcall FSetPOV( dword pIndex, dword pPOV ); 
    long __fastcall FGetRX(); 
    void __fastcall FSetRX( long pRX ); 
    long __fastcall FGetRY(); 
    void __fastcall FSetRY( long pRY ); 
    long __fastcall FGetRZ(); 
    void __fastcall FSetRZ( long pRZ ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 
    long __fastcall FGetSlider( dword pIndex ); 
    void __fastcall FSetSlider( dword pIndex, long pSlider ); 
    long __fastcall FGetX(); 
    void __fastcall FSetX( long pX ); 
    long __fastcall FGetY(); 
    void __fastcall FSetY( long pY ); 
    long __fastcall FGetZ(); 
    void __fastcall FSetZ( long pZ ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DIJOYSTATE* __fastcall FGetInternal_DIJOYSTATE_Ptr(); 
    void __fastcall FSetInternal_DIJOYSTATE( DIJOYSTATE* pDIJOYSTATE ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    dword fSize; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DIJOYSTATE fDIJOYSTATE; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
