#ifndef TDIKeyboardStateH
#define TDIKeyboardStateH

// ==========================================================================
// File:         TDIKeyboardState.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons, website),
//               Brian Austin (some source code, documentation)
// Description:  This file defines the TDIKeyboardState Component 
//
// "TDx_Input_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Input_Library_Defns.H"
#include "TDx_Input_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDIKeyboardState
// Description: The TDIKeyboardState component is used to retrieve data from 
//              a TDx_InputDevice in the form of a 256 byte array. 
//              
//              This component is used with the 
//              TDx_InputDevice::GetDeviceState() method to obtain current 
//              device information, and with the 
//              TDx_InputDevice::SetDataFormat() method to set the devices 
//              data format. 
//              
//              The available key constants are: 
//              
//              On main keyboard area 
//              
//              DIK_ESCAPE     
//              DIK_1  "1" key on the main keyboard area pressed, DIK_NUMPAD1 
//              is the "1" key on the numeric keypad 
//              DIK_2   
//              DIK_3 
//              DIK_4 
//              DIK_5   
//              DIK_6   
//              DIK_7   
//              DIK_8   
//              DIK_9   
//              DIK_0   
//              DIK_MINUS "-" key on main area, use DIK_SUBTRACT for numeric 
//              Keypad "-" 
//              DIK_EQUALS 
//              DIK_BACK  BACKSPACE key   
//              DIK_TAB     
//              DIK_Q     
//              DIK_W     
//              DIK_E     
//              DIK_R     
//              DIK_T     
//              DIK_Y     
//              DIK_U     
//              DIK_I     
//              DIK_O     
//              DIK_P     
//              DIK_LBRACKET   "[" key   
//              DIK_RBRACKET   "]" key   
//              DIK_RETURN  ENTER key on main keyboard area 
//              DIK_LCONTROL  Left CTRL key   
//              DIK_A     
//              DIK_S     
//              DIK_D     
//              DIK_F     
//              DIK_G     
//              DIK_H     
//              DIK_J     
//              DIK_K     
//              DIK_L     
//              DIK_SEMICOLON     
//              DIK_APOSTROPHE     
//              DIK_GRAVE  "`" key   
//              DIK_LSHIFT  Left SHIFT key   
//              DIK_BACKSLASH     
//              DIK_Z     
//              DIK_X     
//              DIK_C     
//              DIK_V     
//              DIK_B     
//              DIK_N     
//              DIK_M     
//              DIK_COMMA     
//              DIK_PERIOD  On main keyboard area, use DIK_DECIMAL for keypad 
//              "." 
//              DIK_SLASH  Forward slash on main keyboard area 
//              DIK_RSHIFT  Right SHIFT key   
//              DIK_MULTIPLY  "*" key on numeric keypad   
//              DIK_LMENU  Left ALT key   
//              DIK_RMENU  Right ALT key   
//              DIK_SPACE  SPACEBAR   
//              DIK_CAPITAL  CAPS LOCK key   
//              DIK_RCONTROL  Right CTRL key   
//              DIK_LWIN  Left Windows key   
//              DIK_RWIN  Right Windows key   
//              
//              Function keys 
//              
//              DIK_F1     
//              DIK_F2     
//              DIK_F3     
//              DIK_F4     
//              DIK_F5     
//              DIK_F6     
//              DIK_F7     
//              DIK_F8     
//              DIK_F9     
//              DIK_F10     
//              DIK_F11     
//              DIK_F12     
//              DIK_F13   
//              DIK_F14   
//              DIK_F15   
//              DIK_SCROLL  SCROLL LOCK 
//              DIK_PAUSE     
//              
//              Edit and Navigation keys 
//              
//              DIK_INSERT     
//              DIK_DELETE     
//              DIK_HOME     
//              DIK_END     
//              DIK_PRIOR  PAGE UP   
//              DIK_NEXT  PAGE DOWN   
//              DIK_UP  UP ARROW   
//              DIK_LEFT  LEFT ARROW   
//              DIK_RIGHT  RIGHT ARROW   
//              DIK_DOWN  DOWN ARROW   
//              
//              Numeric Keypad values 
//              
//              DIK_NUMLOCK     
//              DIK_NUMPAD7     
//              DIK_NUMPAD8     
//              DIK_NUMPAD9     
//              DIK_SUBTRACT  "-" on numeric keypad   
//              DIK_NUMPAD4     
//              DIK_NUMPAD5     
//              DIK_NUMPAD6     
//              DIK_ADD  "+" on numeric keypad   
//              DIK_NUMPAD1     
//              DIK_NUMPAD2     
//              DIK_NUMPAD3     
//              DIK_NUMPAD0     
//              DIK_DECIMAL  "." on numeric keypad   
//              DIK_NUMPADENTER     
//              DIK_DIVIDE  Forward slash on numeric keypad   
//              
//              Japanese Keyboard definitions, do not use for standard 
//              keyboards 
//              
//              DIK_KANA 
//              DIK_CONVERT 
//              DIK_NOCONVERT 
//              DIK_YEN 
//              DIK_NUMPADEQUALS numeric keypad EQUALS (NEC PC98 only) 
//              DIK_CIRCUMFLEX 
//              DIK_AT 
//              DIK_COLON 
//              DIK_UNDERLINE 
//              DIK_KANJI 
//              DIK_STOP 
//              DIK_AX 
//              DIK_UNLABELED 
//              DIK_NUMPADCOMMA COMMA on NEC PC98 numeric keypad only 
//              
//              Other keys 
//              
//              DIK_SYSRQ     
//              DIK_APPS  Application key   
//              
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDIKeyboardState : public TDIData
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDIKeyboardState : public TDIData
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDIKeyboardState component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDIKeyboardState method or 
    //              fget/fset. eg. DI_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DI_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Keys
    // Description: The Keys property is used to store results from the 
    //              TDx_InputDevice::GetDeviceState() method as a 256 byte 
    //              array. 
    //              
    //              When a given key is down, the high-order bit of the 
    //              corresponding byte will be set. 
    //              If the key is up, or does not exist, the bit will be 
    //              clear. 
    // ----------------------------------------------------------------------
    __property byte Keys[dword pIndex] = { read=FGetKeys, write=FSetKeys }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of the 
    //              internal DIKEYBOARDSTATE structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDIKeyboardState's internal DIKEYBOARDSTATE 
    //              structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDIKeyboardState(TComponent* Owner); 
    virtual __fastcall ~TDIKeyboardState(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DIKEYBOARDSTATE* Internal_DIKEYBOARDSTATE_Ptr = { 
    read=FGetInternal_DIKEYBOARDSTATE_Ptr, nodefault }; 

    void __fastcall Internal_DIKEYBOARDSTATE_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    byte __fastcall FGetKeys( dword pIndex ); 
    void __fastcall FSetKeys( dword pIndex, byte pKeys ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DIKEYBOARDSTATE* __fastcall FGetInternal_DIKEYBOARDSTATE_Ptr(); 
    void __fastcall FSetInternal_DIKEYBOARDSTATE( DIKEYBOARDSTATE* 
    pDIKEYBOARDSTATE ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    dword fSize; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DIKEYBOARDSTATE fDIKEYBOARDSTATE; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
