#ifndef TDIPropHeaderH
#define TDIPropHeaderH

// ==========================================================================
// File:         TDIPropHeader.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons, website),
//               Brian Austin (some source code, documentation)
// Description:  This file defines the TDIPropHeader Component 
//
// "TDx_Input_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Input_Library_Defns.H"
#include "TDx_Input_Library_Functions.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDIPropHeader
// Description: The TDIPropHeader component wraps the DirectX DIPROPHEADER 
//              structure which is located at the start of all 
//              TDIProperty-derived components internal structures, in a 
//              structure-in-structure relationship. 
//              
//              These TDIProperty-derived components are TDIPropDWORD, 
//              TDIPropGuidAndPath, TDIPropRange and TDIPropString, which 
//              wrap the various DIPROPDWORD, DIGUIDANDPATH, DIPROPRANGE and 
//              DIPROPSTRING structures. 
//              
//              Each of these internal structures contains a DIPROPHEADER 
//              structure which is used to define the size of the 
//              encapsulating structure and how the data within the 
//              encapsulating structure should be interpreted. This structure 
//              also defines a special-purpose dwSize member, usually 
//              maintained with manual code. 
//              
//              To automatically maintain this dwSize member and to access 
//              all relevant information within these TDIProperty components, 
//              link a TDIPropHeader component using the relevant 
//              TDIPropDWORD::PropHeader, TDIPropGuidAndPath::PropHeader, 
//              TDIPropRange::PropHeader and TDIPropString::PropHeader 
//              properties. 
//              
//              During linking, the TDIPropHeader::Size property is 
//              automatically set to the TDIProperty's Size property. 
//              
//              Example code that sets a property :- 
//              
//                DIPropHeader1->Obj = 0; 
//                DIPropHeader1->How = DIPH_DEVICE; 
//                DIPropDWORD1->PropHeader = DIPropHeader1; 
//                DIPropDWORD1->Data = 10; 
//                Dx_InputDevice1->SetProperty( DIPROP_BUFFERSIZE, 
//              DIPropDWORD1 ); 
//              
//              You can also access the contents of the TDIPropDWORD's 
//              internal DIPROPHEADER structure using standard vcl code like 
//              :- 
//              
//                DIPropDWORD1->PropHeader = DIPropHeader1; 
//                DIPropDWORD1->PropHeader->Obj = 0; 
//                DIPropDWORD1->PropHeader->How = DIPH_DEVICE; 
//                DIPropDWORD1->Data = 10; 
//                Dx_InputDevice1->SetProperty( DIPROP_BUFFERSIZE, 
//              DIPropDWORD1 ); 
//              
//              After linking and setting up the relevant details, pass the 
//              TDIProperty-derived component into 
//              TDx_InputDevice::GetProperty() and 
//              TDx_InputDevice::SetProperty() to perform a range of property 
//              related operations on an input device or object. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDIPropHeader : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDIPropHeader : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    HeaderSize
    // Description: The HeaderSize property defines the size, in bytes, of 
    //              the internal DIPROPHEADER structure used by this 
    //              component. 
    // ----------------------------------------------------------------------
    __property dword HeaderSize = { read=FGetHeaderSize, 
    write=FSetHeaderSize, default=sizeof(DIPROPHEADER) }; 

    // ----------------------------------------------------------------------
    // Property:    How
    // Description: The How property specifies how the TDIPropHeader::Obj 
    //              property should be interpreted. 
    //              The possible values are: 
    //              
    //              DIPH_DEVICE 
    //                - TDIPropHeader::Obj must be zero. 
    //              
    //              DIPH_BYOFFSET 
    //                - TDIPropHeader::Obj represents a byte offset into the 
    //              data format of the object whose property setting is being 
    //              accessed. 
    //              
    //              DIPH_BYID 
    //                - TDIPropHeader::Obj represents an object type/instance 
    //              identifier. The identifier can be found in 
    //              TDIDeviceObjectInstance::Type when it returns from a 
    //              TDx_InputDevice::EnumObjects() method call. 
    // ----------------------------------------------------------------------
    __property dword How = { read=FGetHow, write=FSetHow, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Obj
    // Description: The Obj property identifies the object whose property is 
    //              to be accessed, either as an identifier or a data format 
    //              offset. The interpretation of this property depends on 
    //              the contents of TDIPropHeader::How. 
    // ----------------------------------------------------------------------
    __property dword Obj = { read=FGetObj, write=FSetObj, default=0 }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDIPropHeader component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDIPropHeader method or 
    //              fget/fset. eg. DI_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DI_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: Note: Unlike most Size properties, this property defines 
    //              the size, in bytes, of the internal structure used within 
    //              TDIPropDWORD, TDIPropGuidAndPath, TDIPropRange and 
    //              TDIPropString components that have been linked to this 
    //              TDIPropHeader component. This Size property does not 
    //              define the size in bytes of the internal DIPROPHEADER 
    //              structure used within this component. 
    //              
    //              For a more detailed description of this property, refer 
    //              to the help on the TDIPropHeader component. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDIPropHeader's internal DIPROPHEADER structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDIPropHeader(TComponent* Owner); 
    virtual __fastcall ~TDIPropHeader(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DIPROPHEADER* Internal_DIPROPHEADER_Ptr = { 
    read=FGetInternal_DIPROPHEADER_Ptr, nodefault }; 

    void __fastcall Internal_DIPROPHEADER_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    dword __fastcall FGetHeaderSize(); 
    void __fastcall FSetHeaderSize( dword pHeaderSize ); 
    dword __fastcall FGetHow(); 
    void __fastcall FSetHow( dword pHow ); 
    dword __fastcall FGetObj(); 
    void __fastcall FSetObj( dword pObj ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DIPROPHEADER* __fastcall FGetInternal_DIPROPHEADER_Ptr(); 
    void __fastcall FSetInternal_DIPROPHEADER( DIPROPHEADER* pDIPROPHEADER ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    dword fSize; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DIPROPHEADER fDIPROPHEADER; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
