#ifndef TDIPropRangeH
#define TDIPropRangeH

// ==========================================================================
// File:         TDIPropRange.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons, website),
//               Brian Austin (some source code, documentation)
// Description:  This file defines the TDIPropRange Component 
//
// "TDx_Input_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Input_Library_Defns.H"
#include "TDx_Input_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by TDIPropRange properties.
#include "TDIPropHeader.H"

// --------------------------------------------------------------------------
// ==========================================================================
// Class:       TDIPropRange
// Description: The TDIPropRange component wraps the DirectX DIPROPRANGE 
//              structure which used to access information about the possible 
//              range values of an object. 
//              
//              This component is primarily utilised by the 
//              TDx_InputDevice::GetProperty() and 
//              TDx_InputDevice::SetProperty() methods to access properties 
//              that define a device objects reporting range. 
//              
//              This component also references a TDIPropHeader component. 
//              
//              For some devices you cannot set the range values by calling 
//              TDx_InputDevice::SetProperty(). 
//              When device ranges are unrestricted they will wrap around. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDIPropRange : public TDIProperty
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDIPropRange : public TDIProperty
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Max
    // Description: The Max property defines the upper limit of the devices 
    //              range. 
    //              
    //              This property will be DIPROPRANGE_NOMAX when 
    //              TDx_InputDevice::GetProperty() returns if there are no 
    //              range restrictions on the device. (A mouse axis for 
    //              example) 
    // ----------------------------------------------------------------------
    __property long Max = { read=FGetMax, write=FSetMax, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    Min
    // Description: The Min property defines the lower limit of the devices 
    //              range. 
    //              
    //              This property will be DIPROPRANGE_NOMIN when 
    //              TDx_InputDevice::GetProperty() returns if there are no 
    //              range restrictions on the device. (A mouse axis for 
    //              example) 
    // ----------------------------------------------------------------------
    __property long Min = { read=FGetMin, write=FSetMin, default=0 }; 

    // ----------------------------------------------------------------------
    // Property:    PropHeader
    // Description: The PropHeader property can be used to link the 
    //              TDIPropRange to a TDIPropHeader component using the 
    //              Object Inspector or equivalent runtime code, and is used 
    //              to access data inside the DIPROPHEADER structure that 
    //              exists inside a DIPROPRANGE structure in a 
    //              structure-in-structure relationship. 
    //              
    //              Normally in DirectX, you must setup the entire 
    //              DIPROPRANGE structure including it's DIPROPHEADER 
    //              structure according to specific rules. Upon linking a 
    //              TDIPropHeader component to a TDIPropRange, much of the 
    //              work has already been performed, but you still need to 
    //              setup both the TDIPropHeader::Obj and the 
    //              TDIPropHeader::How properties. 
    //              
    //              For Example :- 
    //              
    //              void __fastcall Form1::Button1Click(TObject* Sender) 
    //              { 
    //              DIPropHeader1->How = DIPH_DEVICE; 
    //              DIPropHeader1->Obj = 0; 
    //              
    //              DIPropRange1->Min = -1000; 
    //              DIPropRange1->Max = 1000; 
    //              DIPropRange1->PropHeader = DIPropHeader1; 
    //              
    //              Dx_InputDevice1->SetProperty( DIPROP_RANGE, DIPropRange1 
    //              ); 
    //              } 
    // ----------------------------------------------------------------------
    __property TDIPropHeader* PropHeader = { read=FGetPropHeader, 
    write=FSetPropHeader, default=NULL }; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDIPropRange component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDIPropRange method or 
    //              fget/fset. eg. DI_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DI_OK }; 

    // ----------------------------------------------------------------------
    // Property:    Size
    // Description: The Size property contains the size in bytes of the 
    //              internal DIPROPRANGE structure. 
    // ----------------------------------------------------------------------
    __property dword Size = { read=FGetSize, write=FSetSize, nodefault }; 

    // ----------------------------------------------------------------------
    // Method:      Clear()
    // Description: The Clear() method can be used to clear the contents of 
    //              the TDIPropRange's internal DIPROPRANGE structure. 
    //              Note: if you have manually linked a structure member to a 
    //              chunk of memory, make sure you release this memory before 
    //              calling Clear(). 
    // ----------------------------------------------------------------------
    virtual void __fastcall Clear(); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDIPropRange(TComponent* Owner); 
    virtual __fastcall ~TDIPropRange(); 

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    __property DIPROPRANGE* Internal_DIPROPRANGE_Ptr = { 
    read=FGetInternal_DIPROPRANGE_Ptr, nodefault }; 

    void __fastcall Internal_DIPROPRANGE_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 
    long __fastcall FGetMax(); 
    void __fastcall FSetMax( long pMax ); 
    long __fastcall FGetMin(); 
    void __fastcall FSetMin( long pMin ); 
    TDIPropHeader* __fastcall FGetPropHeader(); 
    void __fastcall FSetPropHeader( TDIPropHeader* pPropHeader ); 
    dword __fastcall FGetSize(); 
    void __fastcall FSetSize( dword pSize ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Internal Structure Access 
    // ----------------------------------------------------------------------
    DIPROPRANGE* __fastcall FGetInternal_DIPROPRANGE_Ptr(); 
    void __fastcall FSetInternal_DIPROPRANGE( DIPROPRANGE* pDIPROPRANGE ); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    HRESULT fErrorValue; 
    TDIPropHeader* fPropHeader; 
    dword fSize; 

    // ----------------------------------------------------------------------
    // Structure Variables 
    // ----------------------------------------------------------------------
    DIPROPRANGE fDIPROPRANGE; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Error FOnError;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
