// --------------------------------------------------------------------------

// ==========================================================================
// File:         TDx_Input.CPP
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons, website),
//               Brian Austin (some source code, documentation)
// Description:  This file contains the code for the TDx_Input Component 
//
// "TDx_Input_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// #includes ...
// --------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop
// --------------------------------------------------------------------------
#include "TDx_Input.H"
// --------------------------------------------------------------------------
// external classes used by the TDx_Input component.
#include "TDIDeviceInstance.H"
// --------------------------------------------------------------------------
// external classes used by TDx_Input methods.
#include "TDx_InputDevice.H"
// --------------------------------------------------------------------------
#pragma link "TDx_Library_Defns"
#pragma link "TDx_Library_Functions"
#pragma link "TDx_Input_Library_Defns"
#pragma link "TDx_Input_Library_Functions"

// --------------------------------------------------------------------------
// Object Registration...
// --------------------------------------------------------------------------
#if (__BORLANDC__ >= 0x0530) // BCB Version 3+
    #pragma package(smart_init)
#endif
// --------------------------------------------------------------------------
static inline void ValidCtrCheck(TDx_Input*)
{
    new TDx_Input(NULL);
}
// --------------------------------------------------------------------------
namespace Tdx_input
{
    #if (__BORLANDC__ >= 0x0530) // BCB Version 3+
    void __fastcall PACKAGE Register()
    #else
    void __fastcall Register()
    #endif
    {
        TComponentClass classes[1] = {__classid(TDx_Input)};
        RegisterComponents("TDx_Input", classes, 0);
    }
}

// --------------------------------------------------------------------------
// Extra code for Callback Handling
// --------------------------------------------------------------------------
TDx_Input* TDx_Input_OnEnumDevicesOwner = NULL;

// --------------------------------------------------------------------------
// Constructor: TDx_Input::TDx_Input()
// Description: The default constructor for the TDx_Input object.
// --------------------------------------------------------------------------
__fastcall TDx_Input::TDx_Input(TComponent* Owner)
    : TComponent(Owner)
{
fCreated = false;
fErrorValue = DI_OK;
}
// --------------------------------------------------------------------------
// Destructor:  TDx_Input::~TDx_Input()
// Description: The destructor for the TDx_Input object.
// --------------------------------------------------------------------------
__fastcall TDx_Input::~TDx_Input()
{
// destroy internals
if (Created) Destroy();
}
// --------------------------------------------------------------------------
// Property:    Created
// Description: The Created property is true if the internal LPDIRECTINPUT7 
//              used in this component has been successfully created, 
//              otherwise Created is false. 
//              
//              To create the internal LPDIRECTINPUT7, call the 
//              TDx_Input::Create() method. 
// --------------------------------------------------------------------------
bool __fastcall TDx_Input::FGetCreated()
{
return fCreated;
}
// --------------------------------------------------------------------------
void __fastcall TDx_Input::FSetCreated( bool pCreated )
{
fCreated = (pCreated && (fLPDIRECTINPUT7==NULL));
}
// --------------------------------------------------------------------------
// Property:    ErrorValue
// Description: The ErrorValue property contains the last error value 
//              returned from a call to a TDx_Input method or fget/fset. eg. 
//              DI_OK or DDERR_SURFACELOST or TDX_ERROR 
// --------------------------------------------------------------------------
HRESULT __fastcall TDx_Input::FGetErrorValue()
{
return fErrorValue;
}
// --------------------------------------------------------------------------
void __fastcall TDx_Input::FSetErrorValue( HRESULT pErrorValue )
{
fErrorValue = pErrorValue;
}
// --------------------------------------------------------------------------
// Method:      Create()
// Description: The Create() method is used to automatically create the 
//              internal LPDIRECTINPUT7 interface used in the TDx_Input 
//              component and must be called before any methods of the 
//              TDx_Input component will function. 
// --------------------------------------------------------------------------
bool __fastcall TDx_Input::Create()
{
// if the component internals have already been created, exit
if (fCreated)
    {
    if (FOnError) FOnError( this, Name+"::Create()", "TDX_ALREADYCREATED", "The "+Name+"::Create() method was called when the component has already been created successfully." );
    return false;
    }

// create internal IDirectInput interface
fErrorValue = DirectInputCreateEx( HInstance, DIRECTINPUT_VERSION, IID_IDirectInput7, (void**) &fLPDIRECTINPUT7, NULL );

// set success value
if (fErrorValue==DI_OK)
    {
    fCreated = true;
    if (FOnCreate) FOnCreate(this);
    }

// return true if created
return fCreated;
}


// --------------------------------------------------------------------------
// Method:      CreateDevice()
// Description: The CreateDevice() method will create the supplied 
//              TDx_InputDevice component according to a specified GUID. 
//              
//              In DirectX7, the existing IDirectInput::CreateDevice() 
//              function is superceded by a new 
//              IDirectInput::CreateDeviceEx() function.  Calling this method 
//              is equivalent to the IDirectInput7::CreateDeviceEx() 
//              function. 
//              
//              Error values that can be generated are: 
//              
//              DIERR_DEVICENOTREG   
//              DIERR_INVALIDPARAM   
//              DIERR_NOINTERFACE   
//              DIERR_NOTINITIALIZED   
//              DIERR_OUTOFMEMORY 
// Params:      pRefGuid - 
//                The RefGuid parameter defines the GUID of the desired input 
//                device. 
//                
//                This GUID may be one retrieved using TDx_Input::EnumDevices() 
//                or it may be one of the following : 
//                
//                    GUID_SysKeyboard - The default system keyboard. 
//                    GUID_SysMouse - The default system mouse. 
//              pInputDevice - 
//                The InputDevice parameter references TDx_InputDevice 
//                component which will contain a newly created IDirectInput7 
//                interface if the method call succeeds. 
// --------------------------------------------------------------------------
bool __fastcall TDx_Input::CreateDevice( REFGUID pRefGuid, TDx_InputDevice* pInputDevice )
{
// Original Function Definition
// HRESULT CreateDeviceEx( 
//   REFGUID rguid,   
//   REFIID riid, 
//   LPDIRECTINPUTDEVICE7* pvOut, 
//   LPUNKNOWN pUnkOuter 
// );   // note: LPDIRECTINPUT7 really is a LPVOID* 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::CreateDevice()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Check Parameters for Accuracy
if (pInputDevice==NULL)
    {
    fErrorValue = TDX_BADPARAMS;
    if (FOnError) FOnError( this, Name+"::CreateDevice()", "TDX_BADPARAMS", "'pInputDevice' cannot be 'NULL'");
    return false;
    }

// Call Original Function
fErrorValue = fLPDIRECTINPUT7->CreateDeviceEx( pRefGuid, IID_IDirectInputDevice7, (void**) pInputDevice->Internal_LPDIRECTINPUTDEVICE7_Ptr, NULL );

// Translate Data returned from Function
pInputDevice->Internal_LPDIRECTINPUTDEVICE7_Update();

// Handle any Known Results
if (fErrorValue!=DI_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::CreateDevice()", TDx_Input_Library_ErrorString(fErrorValue), TDx_Input_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}

// --------------------------------------------------------------------------
// Method:      Destroy()
// Description: The Destroy() method is used to automatically destroy the 
//              internal LPDIRECTINPUT7 interface used in the TDx_Input 
//              component and should be called when the internal interface is 
//              no longer required. 
//              
//              Note: This method is called by the component's destructor. 
// --------------------------------------------------------------------------
bool __fastcall TDx_Input::Destroy()
{
// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::Destroy()", "TDX_NOTCREATED", "The "+Name+"::Destroy() method was called before being created successfully." );
    return false;
    }

// code that destroys the internal interface
if (FOnDestroy) FOnDestroy(this);

if (fLPDIRECTINPUT7==NULL)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, Name+"::Destroy()", "TDX_ERROR", "The "+Name+" component's internal LPDIRECTINPUT7 is NULL but the TDx_Input component is expecting it to be not NULL. Aborting the Destroy()." );
    return false;
    }

try {
    fLPDIRECTINPUT7->Release();
    }
catch (...)
    {
    fErrorValue = TDX_ERROR;
    if (FOnError) FOnError( this, Name+"::Destroy()", "TDX_ERROR", "An exception has occurred within LPDIRECTINPUT7->Release(). Check that you are destroying all components in reverse order of creation." );
    }

// successful destruction
fLPDIRECTINPUT7 = NULL;
fCreated = false;
return true;
}

// --------------------------------------------------------------------------
// Method:      EnumDevices()
// Description: The EnumDevices() method will enumerate the physical devices 
//              attached and installed on the computer. 
//              
//              Devices not physically present (plugged in) can be enumerated 
//              if drivers for the device are present on the system. 
//              
//              Error values that can be generated are: 
//              
//              DIERR_INVALIDPARAM   
//              DIERR_NOTINITIALIZED 
// Params:      pDeviceType - 
//                The DeviceType parameter defines which device types should be 
//                enumerated. 
//                
//                A setting of 0 will enumerate all device types, otherwise use 
//                a combination of the values below. 
//                
//                Basic Types 
//                
//                DIDEVTYPE_MOUSE - Enumerate basic mice/trackballs. 
//                DIDEVTYPE_KEYBOARD - Enumerate basic keyboards. 
//                DIDEVTYPE_JOYSTICK - Enumerate basic joysticks. 
//                DIDEVTYPE_DEVICE - Enumerate non standard devices. 
//                
//                Mouse Subtypes 
//                
//                DIDEVTYPEMOUSE_UNKNOWN - Enumerate mice with unknown 
//                subtypes. 
//                DIDEVTYPEMOUSE_TRADITIONAL - Enumerate traditional mice. 
//                DIDEVTYPEMOUSE_FINGERSTICK - Enumerate fingersticks. 
//                DIDEVTYPEMOUSE_TOUCHPAD - Enumerate touchpads. 
//                DIDEVTYPEMOUSE_TRACKBALL - Enumerate trackballs. 
//                
//                Keyboard Subtypes 
//                
//                DIDEVTYPEKEYBOARD_UNKNOWN - Enumerate keyboards with unknown 
//                subtypes. 
//                DIDEVTYPEKEYBOARD_PCXT - Enumerate IBM PC/XT 83-key 
//                keyboards. 
//                DIDEVTYPEKEYBOARD_OLIVETTI - Enumerate Olivetti 102-key 
//                keyboards. 
//                DIDEVTYPEKEYBOARD_PCAT - Enumerate IBM PC/AT 84-key 
//                keyboards. 
//                DIDEVTYPEKEYBOARD_PCENH - Enumerate IBM PC Enhanced 
//                101/102-key or Microsoft Natural keyboards. 
//                DIDEVTYPEKEYBOARD_NOKIA1050 - Enumerate Nokia 1050 keyboards. 
//                DIDEVTYPEKEYBOARD_NOKIA9140 - Enumerate Nokia 9140 keyboards. 
//                DIDEVTYPEKEYBOARD_NEC98 - Enumerate Japanese NEC PC98 
//                keyboards. 
//                DIDEVTYPEKEYBOARD_NEC98LAPTOP - Enumerate Japanese NEC PC98 
//                laptop keyboards. 
//                DIDEVTYPEKEYBOARD_NEC98106 - Enumerate Japanese NEC PC98 
//                106-key keyboards. 
//                DIDEVTYPEKEYBOARD_JAPAN106 - Enumerate Japanese 106-key 
//                keyboards. 
//                DIDEVTYPEKEYBOARD_JAPANAX - Enumerate Japanese AX keyboards. 
//                DIDEVTYPEKEYBOARD_J3100 - Enumerate Japanese J3100 keyboards. 
//                
//                Joystick Subtypes 
//                
//                DIDEVTYPEJOYSTICK_UNKNOWN - Enumerate joysticks with unknown 
//                subtypes. 
//                DIDEVTYPEJOYSTICK_TRADITIONAL - Enumerate traditional 
//                joysticks. 
//                DIDEVTYPEJOYSTICK_FLIGHTSTICK - Enumerate flightsticks. 
//                DIDEVTYPEJOYSTICK_GAMEPAD - Enumerate gamepads. 
//                DIDEVTYPEJOYSTICK_RUDDER - Enumerate rudders. 
//                DIDEVTYPEJOYSTICK_WHEEL - Enumerate steering wheels. 
//                DIDEVTYPEJOYSTICK_HEADTRACKER - Enumerate headtrackers. 
//                
//                HID 
//                
//                DIDEVTYPE_HID - Enumerate devices that uses the Human 
//                Interface Device (HID) protocol. 
//              pContext - 
//                The Context parameter should hold an application defined 
//                32-bit value to be passed to the callback function each time 
//                it is called. 
//              pFlags - 
//                The Flags parameter defines flags indicating which devices 
//                should be enumerated. 
//                The described effect applies when the flag is set. 
//                Flags:
//                  DIEDFL_ALLDEVICES -
//                    All devices installed on the system are to be 
//                    enumerated. 
//                    This is the default setting. 
//                  DIEDFL_ATTACHEDONLY -
//                    Only devices that are both attached and installed on 
//                    the system are to be enumerated. 
//                  DIEDFL_FORCEFEEDBACK -
//                    Only devices that support force feedback are to be 
//                    enumerated. 
//                  DIEDFL_INCLUDEALIASES -
//                    Devices that are aliases for other devices are also to 
//                    be enumerated. 
//                  DIEDFL_INCLUDEPHANTOMS -
//                    Devices that are phantoms (placeholder devices) are 
//                    also to be enumerated. 
// --------------------------------------------------------------------------
bool __fastcall TDx_Input::EnumDevices( dword pDeviceType, void* pContext, dword pFlags )
{
// Original Function Definition
// HRESULT EnumDevices( 
//   DWORD dwDevType,               
//   LPDIENUMCALLBACK lpCallback,   
//   LPVOID pvRef,                 
//   DWORD dwFlags                 
// ); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::EnumDevices()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Call Original Function
TDx_Input_OnEnumDevicesOwner = this;
fErrorValue = fLPDIRECTINPUT7->EnumDevices( (DWORD) pDeviceType, Internal_EnumDevicesCallback, (LPVOID) pContext, (DWORD) pFlags );
TDx_Input_OnEnumDevicesOwner = NULL;

// Handle any Known Results
if (fErrorValue!=DI_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::EnumDevices()", TDx_Input_Library_ErrorString(fErrorValue), TDx_Input_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}


// --------------------------------------------------------------------------
// Method:      FindDevice()
// Description: The FindDevice() method will retrieve the instance GUID of a 
//              newly attached device after the application has recieved a 
//              Win32 device management notification. 
//              
//              More information about device arrival notifications can be 
//              found by looking up the DBT_DEVICEARRIVAL event in the Win32 
//              SDK. 
//              
//              Error values that can be generated are: 
//              
//              DIERR_DEVICENOTREG 
// Params:      pClass - 
//                The Class parameter identifies the device class for the 
//                device being located. 
//                
//                The device class GUID can be obtained from the device arrival 
//                notification. 
//              pName - 
//                The Name parameter defines the name of the device. 
//                The device name can be obtained from the device arrival 
//                notification. 
//              pInstance - 
//                The Instance parameter references a GUID for storing the 
//                retrieved device instance GUID if the method finds the device 
//                and returns successfully. 
//                
//                Once obtained, this GUID can be passed to 
//                TDx_Input::CreateDevice() to create an interface for the 
//                device. 
//                
//                If no device exists, the Instance parameter will reference a 
//                NULL GUID. 
// --------------------------------------------------------------------------
bool __fastcall TDx_Input::FindDevice( REFGUID pClass, AnsiString pName, GUID* pInstance )
{
// Original Function Definition
// HRESULT FindDevice( 
//   REFGUID rguidClass, 
//   LPCTSTR ptszName, 
//   GUID* pguidInstance 
// ); 
// 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::FindDevice()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Check Parameters for Accuracy
if (pInstance==NULL)
    {
    fErrorValue = TDX_BADPARAMS;
    if (FOnError) FOnError( this, Name+"::FindDevice()", "TDX_BADPARAMS", "'pInstance' cannot be 'NULL'");
    return false;
    }

// Translate Parameters before calling Original Function
AnsiString TempName = pName;

// Call Original Function
fErrorValue = fLPDIRECTINPUT7->FindDevice( pClass, (LPCTSTR) TempName.c_str(), pInstance );

// Handle any Known Results
if (fErrorValue!=DI_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::FindDevice()", TDx_Input_Library_ErrorString(fErrorValue), TDx_Input_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}


// --------------------------------------------------------------------------
// Method:      GetDeviceStatus()
// Description: The GetDeviceStatus() method will return the status of the 
//              specified device. 
//              
//              Error Values that can be generated are: 
//              
//              DIERR_GENERIC   
//              DIERR_INVALIDPARAM   
//              DIERR_NOTINITIALIZED   
// Params:      pInstance - 
//                The Instance parameter identifies the device instance whose 
//                status is to be checked. 
// --------------------------------------------------------------------------
HRESULT __fastcall TDx_Input::GetDeviceStatus( REFGUID pInstance )
{
// Original Function Definition
// HRESULT GetDeviceStatus( 
//   REFGUID rguidInstance   
// ); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::GetDeviceStatus()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Call Original Function
fErrorValue = fLPDIRECTINPUT7->GetDeviceStatus( pInstance );

// Handle any Known Results
if ((fErrorValue!=DI_OK) &&
    (fErrorValue!=DI_NOTATTACHED ))
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::GetDeviceStatus()", TDx_Input_Library_ErrorString(fErrorValue), TDx_Input_Library_ErrorMessage(fErrorValue) );
    return fErrorValue;
    }

// Success!
return fErrorValue;
}


// --------------------------------------------------------------------------
// Method:      Internal_EnumDevicesCallback()
// Description: The Internal_EnumDevicesCallback() method is used internally 
//              by the TDx_Input_Library to redirect the DirectX callback's 
//              triggered during the EnumDevices()  method call to a 
//              user-configurable OnEnumDevices() event. 
// Params:      pDeviceInstance - 
//                The DeviceInstance parameter references the DirectX-supplied 
//                DIDEVICEINSTANCE structure that is currently being 
//                enumerated. 
//              pContext - 
//                The Context parameter is a user defined value passed to the 
//                calling function and sent to this method during each 
//                enumeration. 
// --------------------------------------------------------------------------
BOOL __stdcall TDx_Input::Internal_EnumDevicesCallback( LPCDIDEVICEINSTANCE pDeviceInstance, LPVOID pContext )
{
// Original Function Definition
// BOOL CALLBACK DIEnumDevicesProc(
//   LPCDIDEVICEINSTANCE lpddi,
//   LPVOID pvRef
// );

// Translate parameters
TDIDeviceInstance* device_instance = new TDIDeviceInstance(NULL);
if (device_instance==NULL)
    {
    TDx_Input_OnEnumDevicesOwner->fErrorValue = DIERR_OUTOFMEMORY;
    if (TDx_Input_OnEnumDevicesOwner->FOnError) TDx_Input_OnEnumDevicesOwner->FOnError( TDx_Input_OnEnumDevicesOwner, TDx_Input_OnEnumDevicesOwner->Name+"::Internal_EnumDevices()", TDx_Input_Library_ErrorString(TDx_Input_OnEnumDevicesOwner->fErrorValue), TDx_Input_Library_ErrorMessage(TDx_Input_OnEnumDevicesOwner->fErrorValue) );
    }
CopyMemory( device_instance->Internal_DIDEVICEINSTANCE_Ptr, pDeviceInstance,pDeviceInstance->dwSize);

// Call Event
bool finished = false;
if (TDx_Input_OnEnumDevicesOwner->OnEnumDevices)
    TDx_Input_OnEnumDevicesOwner->OnEnumDevices( TDx_Input_OnEnumDevicesOwner, device_instance, pContext, finished );

// cleanup
delete device_instance;

// Finished!
if (finished) return DIENUM_STOP;
return DIENUM_CONTINUE;
}

// --------------------------------------------------------------------------
// Method:      RunControlPanel()
// Description: The RunControlPanel() method will open the Windows control 
//              panel to allow the user to install or modify input devices. 
//              
//              Error Values that can be generated are: 
//              
//              DIERR_INVALIDPARAM   
//              DIERR_NOTINITIALIZED 
// Params:      pOwner - 
//                The Owner parameter defines the handle of the parent window 
//                of the control panel user interface. 
//                
//                Setting this parameter to NULL results in no parent window 
//                being used. 
// --------------------------------------------------------------------------
bool __fastcall TDx_Input::RunControlPanel( HWND pOwner )
{
// Original Function Definition
// HRESULT RunControlPanel( 
//   HWND hwndOwner,   
//   DWORD dwFlags     
// ); 

// if the component internals are not already created, exit
if (!fCreated)
    {
    fErrorValue = TDX_NOTCREATED;
    if (FOnError) FOnError( this, Name+"::RunControlPanel()", "TDX_NOTCREATED", "The "+Name+" component has not been created successfully." );
    return false;
    }

// Call Original Function
fErrorValue = fLPDIRECTINPUT7->RunControlPanel( pOwner, 0 );

// Handle any Known Results
if (fErrorValue!=DI_OK)
    {
    // Failure.
    if (FOnError) FOnError( this, Name+"::RunControlPanel()", TDx_Input_Library_ErrorString(fErrorValue), TDx_Input_Library_ErrorMessage(fErrorValue) );
    return false;
    }

// Success!
return true;
}


// --------------------------------------------------------------------------
// Internal Interface Access
// --------------------------------------------------------------------------
LPDIRECTINPUT7 __fastcall TDx_Input::FGetInternal_LPDIRECTINPUT7()
{
return fLPDIRECTINPUT7;
}
// --------------------------------------------------------------------------
LPDIRECTINPUT7* __fastcall TDx_Input::FGetInternal_LPDIRECTINPUT7_Ptr()
{
return &fLPDIRECTINPUT7;
}
// --------------------------------------------------------------------------
void __fastcall TDx_Input::FSetInternal_LPDIRECTINPUT7( LPDIRECTINPUT7 pLPDIRECTINPUT7 )
{
if (!fCreated)
    {
    fLPDIRECTINPUT7 = pLPDIRECTINPUT7;
    fCreated = (fLPDIRECTINPUT7!=NULL);
    }
}
// --------------------------------------------------------------------------
void __fastcall TDx_Input::Internal_LPDIRECTINPUT7_Update()
{
fCreated = (fLPDIRECTINPUT7!=NULL);
}
// --------------------------------------------------------------------------
LPDIRECTINPUT __fastcall TDx_Input::FGetInternal_LPDIRECTINPUT()
{
return fLPDIRECTINPUT;
}
// --------------------------------------------------------------------------
LPDIRECTINPUT* __fastcall TDx_Input::FGetInternal_LPDIRECTINPUT_Ptr()
{
return &fLPDIRECTINPUT;
}
// --------------------------------------------------------------------------
void __fastcall TDx_Input::FSetInternal_LPDIRECTINPUT( LPDIRECTINPUT pLPDIRECTINPUT )
{
if (!fCreated)
    {
    fLPDIRECTINPUT = pLPDIRECTINPUT;
    fCreated = (fLPDIRECTINPUT!=NULL);
    }
}
// --------------------------------------------------------------------------
void __fastcall TDx_Input::Internal_LPDIRECTINPUT_Update()
{
fCreated = (fLPDIRECTINPUT7!=NULL);
}

// --------------------------------------------------------------------------
