#ifndef TDx_InputH
#define TDx_InputH

// ==========================================================================
// File:         TDx_Input.H
// Authors:      BCB_Code_Generator v1.70,
//               Darren Dwyer (source code, documentation, demos, website),
//               Hugh Edwards (documentation, icons, website),
//               Brian Austin (some source code, documentation)
// Description:  This file defines the TDx_Input Component 
//
// "TDx_Input_Library v1.70"
// (c) 2003 BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

// --------------------------------------------------------------------------
// #includes
// --------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>
// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Input_Library_Defns.H"
#include "TDx_Input_Library_Functions.H"
// --------------------------------------------------------------------------
// external classes used by the TDx_Input component.
class TDIDeviceInstance;
// --------------------------------------------------------------------------
// external classes used by TDx_Input methods.
class TDx_InputDevice;

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// Definition for the OnEnumDevices callback event.
typedef	__fastcall void	(__closure*          TDx_EnumDevices) 
                        ( TObject*           Sender, 
                          TDIDeviceInstance* DeviceInstance, 
                          void*              Context, 
                          bool&              Finished ); 

// ==========================================================================
// Class:       TDx_Input
// Description: The TDx_Input component wraps the DirectX IDirectInput7 
//              interface which enumerates, creates and retrieves the status 
//              of TDx_Input devices and can also be used to launch the 
//              Windows control panel. 
//              
//              To create an IDirectInput7 interface, use the 
//              TDx_Input::Create() method and to destroy the interface, use 
//              the TDx_Input::Destroy() method. 
//              
//              Use the TDx_Input::OnCreate() event and 
//              TDx_Input::OnDestroy() events to simplify the creation and 
//              destruction of dependent objects. 
//              
//              For example, in the TDx_Input::OnCreate() event, you could 
//              retrieve the TDx_Input interface's capabilities, create a 
//              dependant TDx_InputDevice, and so on. Then in the 
//              TDx_Input::OnDestroy() event, place code to destroy the same 
//              TDx_InputDevice and deallocate memory used to hold the 
//              capabilities (if required). 
//              
//              Once created, call any of the TDx_Input methods and handle 
//              any error results using the TDx_Input::OnError() event and 
//              the TDx_Input::ErrorValue property. 
// ==========================================================================
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
class PACKAGE TDx_Input : public TComponent
{
// --------------------------------------------------------------------------
#else // BCB Version 1
class TDx_Input : public TComponent
{
#endif
// ==========================================================================
__published:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Event:       OnCreate()
    // Description: The OnCreate() event is triggered by the 
    //              TDx_Input::Create() method after it has successfully 
    //              created the internal LPDIRECTINPUT7 used within the 
    //              TDx_Input component. 
    //              
    //              Note that since this event is only called upon successful 
    //              creation, this event is an ideal place for code that is 
    //              to be activated only once, immediately upon the 
    //              successful creation of the LPDIRECTINPUT7 interface. 
    //              
    //              For example, you could retrieve the TDx_Input component's 
    //              capabilities, Create() dependant components, allocate 
    //              memory, etc. 
    //              
    //              When used in conjunction with the TDx_Input::OnDestroy() 
    //              event, you can fully automate and insulate your code on a 
    //              per-component basis. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnCreate = {read=FOnCreate, write=FOnCreate, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnDestroy()
    // Description: The OnDestroy() event is triggered by the 
    //              TDx_Input::Destroy() method, just before the TDx_Input 
    //              internal LPDIRECTINPUT7 interface is destroyed. 
    //              
    //              Note: This event is an ideal place to place code that 
    //              reverses the results of the TDx_Input::OnCreate() event. 
    //              For example, assume you have already setup the 
    //              TDx_Input::OnCreate() event to retrieve the component's 
    //              capabilities, Create() a dependant component and allocate 
    //              some memory; 
    //              You can then fully automate and insulate your code on a 
    //              per-component basis by placing code in the 
    //              TDx_Input::OnDestroy() event to deallocate memory, 
    //              Destroy() the dependant components and, when required, 
    //              destroy the capabilities component. 
    // ----------------------------------------------------------------------
    __property TDx_Event OnDestroy = {read=FOnDestroy, write=FOnDestroy, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Event:       OnError()
    // Description: The OnError event is called when an error occurs in the 
    //              TDx_Input component. 
    // ----------------------------------------------------------------------
    __property TDx_Error OnError = {read=FOnError, write=FOnError, 
    nodefault}; 

    // ----------------------------------------------------------------------
    // Callback:    OnEnumDevices()
    // Description: The OnEnumDevices Callback is called for each device 
    //              enumerated by TDx_Input::EnumDevices(). 
    // ----------------------------------------------------------------------
    __property TDx_EnumDevices OnEnumDevices = {read=FOnEnumDevices, 
    write=FOnEnumDevices, nodefault}; 

// ==========================================================================
public:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property:    Created
    // Description: The Created property is true if the internal 
    //              LPDIRECTINPUT7 used in this component has been 
    //              successfully created, otherwise Created is false. 
    //              
    //              To create the internal LPDIRECTINPUT7, call the 
    //              TDx_Input::Create() method. 
    // ----------------------------------------------------------------------
    __property bool Created = { read=FGetCreated, write=FSetCreated, 
    default=false }; 

    // ----------------------------------------------------------------------
    // Property:    ErrorValue
    // Description: The ErrorValue property contains the last error value 
    //              returned from a call to a TDx_Input method or fget/fset. 
    //              eg. DI_OK or DDERR_SURFACELOST or TDX_ERROR 
    // ----------------------------------------------------------------------
    __property HRESULT ErrorValue = { read=FGetErrorValue, 
    write=FSetErrorValue, default=DI_OK }; 

    // ----------------------------------------------------------------------
    // Method:      Create()
    // Description: The Create() method is used to automatically create the 
    //              internal LPDIRECTINPUT7 interface used in the TDx_Input 
    //              component and must be called before any methods of the 
    //              TDx_Input component will function. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Create(); 

    // ----------------------------------------------------------------------
    // Method:      CreateDevice()
    // Description: The CreateDevice() method will create the supplied 
    //              TDx_InputDevice component according to a specified GUID. 
    //              
    //              In DirectX7, the existing IDirectInput::CreateDevice() 
    //              function is superceded by a new 
    //              IDirectInput::CreateDeviceEx() function.  Calling this 
    //              method is equivalent to the 
    //              IDirectInput7::CreateDeviceEx() function. 
    //              
    //              Error values that can be generated are: 
    //              
    //              DIERR_DEVICENOTREG   
    //              DIERR_INVALIDPARAM   
    //              DIERR_NOINTERFACE   
    //              DIERR_NOTINITIALIZED   
    //              DIERR_OUTOFMEMORY 
    // Params:      pRefGuid - 
    //                The RefGuid parameter defines the GUID of the desired 
    //                input device. 
    //                
    //                This GUID may be one retrieved using 
    //                TDx_Input::EnumDevices() or it may be one of the 
    //                following : 
    //                
    //                    GUID_SysKeyboard - The default system keyboard. 
    //                    GUID_SysMouse - The default system mouse. 
    //              pInputDevice - 
    //                The InputDevice parameter references TDx_InputDevice 
    //                component which will contain a newly created 
    //                IDirectInput7 interface if the method call succeeds. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall CreateDevice( REFGUID pRefGuid, TDx_InputDevice* 
    pInputDevice ); 

    // ----------------------------------------------------------------------
    // Method:      Destroy()
    // Description: The Destroy() method is used to automatically destroy the 
    //              internal LPDIRECTINPUT7 interface used in the TDx_Input 
    //              component and should be called when the internal 
    //              interface is no longer required. 
    //              
    //              Note: This method is called by the component's 
    //              destructor. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall Destroy(); 

    // ----------------------------------------------------------------------
    // Method:      EnumDevices()
    // Description: The EnumDevices() method will enumerate the physical 
    //              devices attached and installed on the computer. 
    //              
    //              Devices not physically present (plugged in) can be 
    //              enumerated if drivers for the device are present on the 
    //              system. 
    //              
    //              Error values that can be generated are: 
    //              
    //              DIERR_INVALIDPARAM   
    //              DIERR_NOTINITIALIZED 
    // Params:      pDeviceType - 
    //                The DeviceType parameter defines which device types 
    //                should be enumerated. 
    //                
    //                A setting of 0 will enumerate all device types, otherwise 
    //                use a combination of the values below. 
    //                
    //                Basic Types 
    //                
    //                DIDEVTYPE_MOUSE - Enumerate basic mice/trackballs. 
    //                DIDEVTYPE_KEYBOARD - Enumerate basic keyboards. 
    //                DIDEVTYPE_JOYSTICK - Enumerate basic joysticks. 
    //                DIDEVTYPE_DEVICE - Enumerate non standard devices. 
    //                
    //                Mouse Subtypes 
    //                
    //                DIDEVTYPEMOUSE_UNKNOWN - Enumerate mice with unknown 
    //                subtypes. 
    //                DIDEVTYPEMOUSE_TRADITIONAL - Enumerate traditional mice. 
    //                DIDEVTYPEMOUSE_FINGERSTICK - Enumerate fingersticks. 
    //                DIDEVTYPEMOUSE_TOUCHPAD - Enumerate touchpads. 
    //                DIDEVTYPEMOUSE_TRACKBALL - Enumerate trackballs. 
    //                
    //                Keyboard Subtypes 
    //                
    //                DIDEVTYPEKEYBOARD_UNKNOWN - Enumerate keyboards with 
    //                unknown subtypes. 
    //                DIDEVTYPEKEYBOARD_PCXT - Enumerate IBM PC/XT 83-key 
    //                keyboards. 
    //                DIDEVTYPEKEYBOARD_OLIVETTI - Enumerate Olivetti 102-key 
    //                keyboards. 
    //                DIDEVTYPEKEYBOARD_PCAT - Enumerate IBM PC/AT 84-key 
    //                keyboards. 
    //                DIDEVTYPEKEYBOARD_PCENH - Enumerate IBM PC Enhanced 
    //                101/102-key or Microsoft Natural keyboards. 
    //                DIDEVTYPEKEYBOARD_NOKIA1050 - Enumerate Nokia 1050 
    //                keyboards. 
    //                DIDEVTYPEKEYBOARD_NOKIA9140 - Enumerate Nokia 9140 
    //                keyboards. 
    //                DIDEVTYPEKEYBOARD_NEC98 - Enumerate Japanese NEC PC98 
    //                keyboards. 
    //                DIDEVTYPEKEYBOARD_NEC98LAPTOP - Enumerate Japanese NEC 
    //                PC98 laptop keyboards. 
    //                DIDEVTYPEKEYBOARD_NEC98106 - Enumerate Japanese NEC PC98 
    //                106-key keyboards. 
    //                DIDEVTYPEKEYBOARD_JAPAN106 - Enumerate Japanese 106-key 
    //                keyboards. 
    //                DIDEVTYPEKEYBOARD_JAPANAX - Enumerate Japanese AX 
    //                keyboards. 
    //                DIDEVTYPEKEYBOARD_J3100 - Enumerate Japanese J3100 
    //                keyboards. 
    //                
    //                Joystick Subtypes 
    //                
    //                DIDEVTYPEJOYSTICK_UNKNOWN - Enumerate joysticks with 
    //                unknown subtypes. 
    //                DIDEVTYPEJOYSTICK_TRADITIONAL - Enumerate traditional 
    //                joysticks. 
    //                DIDEVTYPEJOYSTICK_FLIGHTSTICK - Enumerate flightsticks. 
    //                DIDEVTYPEJOYSTICK_GAMEPAD - Enumerate gamepads. 
    //                DIDEVTYPEJOYSTICK_RUDDER - Enumerate rudders. 
    //                DIDEVTYPEJOYSTICK_WHEEL - Enumerate steering wheels. 
    //                DIDEVTYPEJOYSTICK_HEADTRACKER - Enumerate headtrackers. 
    //                
    //                HID 
    //                
    //                DIDEVTYPE_HID - Enumerate devices that uses the Human 
    //                Interface Device (HID) protocol. 
    //              pContext - 
    //                The Context parameter should hold an application defined 
    //                32-bit value to be passed to the callback function each 
    //                time it is called. 
    //              pFlags - 
    //                The Flags parameter defines flags indicating which 
    //                devices should be enumerated. 
    //                The described effect applies when the flag is set. 
    //                Flags:
    //                  DIEDFL_ALLDEVICES -
    //                    All devices installed on the system are to be 
    //                    enumerated. 
    //                    This is the default setting. 
    //                  DIEDFL_ATTACHEDONLY -
    //                    Only devices that are both attached and installed 
    //                    on the system are to be enumerated. 
    //                  DIEDFL_FORCEFEEDBACK -
    //                    Only devices that support force feedback are to be 
    //                    enumerated. 
    //                  DIEDFL_INCLUDEALIASES -
    //                    Devices that are aliases for other devices are also 
    //                    to be enumerated. 
    //                  DIEDFL_INCLUDEPHANTOMS -
    //                    Devices that are phantoms (placeholder devices) are 
    //                    also to be enumerated. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall EnumDevices( dword pDeviceType, void* pContext, 
    dword pFlags ); 

    // ----------------------------------------------------------------------
    // Method:      FindDevice()
    // Description: The FindDevice() method will retrieve the instance GUID 
    //              of a newly attached device after the application has 
    //              recieved a Win32 device management notification. 
    //              
    //              More information about device arrival notifications can 
    //              be found by looking up the DBT_DEVICEARRIVAL event in the 
    //              Win32 SDK. 
    //              
    //              Error values that can be generated are: 
    //              
    //              DIERR_DEVICENOTREG 
    // Params:      pClass - 
    //                The Class parameter identifies the device class for the 
    //                device being located. 
    //                
    //                The device class GUID can be obtained from the device 
    //                arrival notification. 
    //              pName - 
    //                The Name parameter defines the name of the device. 
    //                The device name can be obtained from the device arrival 
    //                notification. 
    //              pInstance - 
    //                The Instance parameter references a GUID for storing the 
    //                retrieved device instance GUID if the method finds the 
    //                device and returns successfully. 
    //                
    //                Once obtained, this GUID can be passed to 
    //                TDx_Input::CreateDevice() to create an interface for the 
    //                device. 
    //                
    //                If no device exists, the Instance parameter will 
    //                reference a NULL GUID. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall FindDevice( REFGUID pClass, AnsiString pName, 
    GUID* pInstance ); 

    // ----------------------------------------------------------------------
    // Method:      GetDeviceStatus()
    // Description: The GetDeviceStatus() method will return the status of 
    //              the specified device. 
    //              
    //              Error Values that can be generated are: 
    //              
    //              DIERR_GENERIC   
    //              DIERR_INVALIDPARAM   
    //              DIERR_NOTINITIALIZED   
    // Params:      pInstance - 
    //                The Instance parameter identifies the device instance 
    //                whose status is to be checked. 
    // ----------------------------------------------------------------------
    virtual HRESULT __fastcall GetDeviceStatus( REFGUID pInstance ); 

    // ----------------------------------------------------------------------
    // Method:      RunControlPanel()
    // Description: The RunControlPanel() method will open the Windows 
    //              control panel to allow the user to install or modify 
    //              input devices. 
    //              
    //              Error Values that can be generated are: 
    //              
    //              DIERR_INVALIDPARAM   
    //              DIERR_NOTINITIALIZED 
    // Params:      pOwner - 
    //                The Owner parameter defines the handle of the parent 
    //                window of the control panel user interface. 
    //                
    //                Setting this parameter to NULL results in no parent 
    //                window being used. 
    // ----------------------------------------------------------------------
    virtual bool __fastcall RunControlPanel( HWND pOwner ); 

    // ----------------------------------------------------------------------
    // Constructor() and Destructor()
    // ----------------------------------------------------------------------
    __fastcall TDx_Input(TComponent* Owner); 
    virtual __fastcall ~TDx_Input(); 

    // ----------------------------------------------------------------------
    // The following properties and methods are used internally by 
    // TDx_Input_Library and should not be used. 
    // ----------------------------------------------------------------------
    __property LPDIRECTINPUT7 Internal_LPDIRECTINPUT7 = { 
    read=FGetInternal_LPDIRECTINPUT7, write=FSetInternal_LPDIRECTINPUT7, 
    nodefault }; 
    __property LPDIRECTINPUT7* Internal_LPDIRECTINPUT7_Ptr = { 
    read=FGetInternal_LPDIRECTINPUT7_Ptr, nodefault }; 
    void __fastcall Internal_LPDIRECTINPUT7_Update(); 
    __property LPDIRECTINPUT Internal_LPDIRECTINPUT = { 
    read=FGetInternal_LPDIRECTINPUT, write=FSetInternal_LPDIRECTINPUT, 
    nodefault }; 
    __property LPDIRECTINPUT* Internal_LPDIRECTINPUT_Ptr = { 
    read=FGetInternal_LPDIRECTINPUT_Ptr, nodefault }; 
    void __fastcall Internal_LPDIRECTINPUT_Update(); 

// ==========================================================================
protected:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Property Access Methods
    // ----------------------------------------------------------------------
    bool __fastcall FGetCreated(); 
    void __fastcall FSetCreated( bool pCreated ); 
    HRESULT __fastcall FGetErrorValue(); 
    void __fastcall FSetErrorValue( HRESULT pErrorValue ); 

// ==========================================================================
private:
// ==========================================================================

    // ----------------------------------------------------------------------
    // Method:   Internal_EnumDevicesCallback()
    // Description: The Internal_EnumDevicesCallback() method is used 
    //              internally by the TDx_Input_Library to redirect the 
    //              DirectX callback's triggered during the EnumDevices() 
    //               method call to a user-configurable OnEnumDevices() 
    //              event. 
    // Params:      pDeviceInstance - 
    //                The DeviceInstance parameter references the 
    //                DirectX-supplied DIDEVICEINSTANCE structure that is 
    //                currently being enumerated. 
    //              pContext - 
    //                The Context parameter is a user defined value passed to 
    //                the calling function and sent to this method during each 
    //                enumeration. 
    // ----------------------------------------------------------------------
    static BOOL __stdcall Internal_EnumDevicesCallback( LPCDIDEVICEINSTANCE 
    pDeviceInstance, LPVOID pContext ); 

    // ----------------------------------------------------------------------
    // Internal Interface Access
    // ----------------------------------------------------------------------
    LPDIRECTINPUT7 __fastcall FGetInternal_LPDIRECTINPUT7(); 
    void __fastcall FSetInternal_LPDIRECTINPUT7( LPDIRECTINPUT7 
    pLPDIRECTINPUT7 ); 
    LPDIRECTINPUT7* __fastcall FGetInternal_LPDIRECTINPUT7_Ptr(); 
    LPDIRECTINPUT __fastcall FGetInternal_LPDIRECTINPUT(); 
    void __fastcall FSetInternal_LPDIRECTINPUT( LPDIRECTINPUT pLPDIRECTINPUT 
    ); 
    LPDIRECTINPUT* __fastcall FGetInternal_LPDIRECTINPUT_Ptr(); 

    // ----------------------------------------------------------------------
    // Property Variables 
    // ----------------------------------------------------------------------
    bool fCreated; 
    HRESULT fErrorValue; 

    // ----------------------------------------------------------------------
    // Interface Variables 
    // ----------------------------------------------------------------------
    LPDIRECTINPUT7 fLPDIRECTINPUT7; 
    LPDIRECTINPUT fLPDIRECTINPUT; 

    // ----------------------------------------------------------------------
    // Event Variables
    // ----------------------------------------------------------------------
    TDx_Event FOnCreate;
    TDx_Event FOnDestroy;
    TDx_Error FOnError;

    // ----------------------------------------------------------------------
    // Callback Variables
    // ----------------------------------------------------------------------
    TDx_EnumDevices FOnEnumDevices;

};
// --------------------------------------------------------------------------
#endif
// --------------------------------------------------------------------------
