
// ==========================================================================
// File:        TDx_Input_Library_Functions.CPP
// Description: This file implements standard functions used with the 
//              TDx_Input_Library components.
//
// "TDx_Input_Library"
// (c) 2002, BCB-Tools.com Pty. Ltd., Sydney, Australia.
// All Rights Reserved.
//
// Refer to the 'Licence.Txt' file for licencing & copyright information.
// ==========================================================================

#include <vcl\vcl.h>
#pragma hdrstop

// --------------------------------------------------------------------------
#include "TDx_Library_Defns.H"
#include "TDx_Library_Functions.H"
// --------------------------------------------------------------------------
#include "TDx_Input_Library_Defns.H"
#include "TDx_Input_Library_Functions.H"

// --------------------------------------------------------------------------
// Func:            TDx_Input_Library_ErrorString()
// Description:     Use this function to return the name of the error.
// --------------------------------------------------------------------------
AnsiString __fastcall TDx_Input_Library_ErrorString(int Error)
{
AnsiString str = TDx_Library_ErrorString(Error);
if (str!="") return str;
switch (Error)
    {
    case -1: str = "TDX_ERROR"; break;
    case DI_OK:	                       str = "DI_OK"; break;
    case DI_BUFFEROVERFLOW:	       str = "TDI_BUFFEROVERFLOW"; break; 
    case DI_DOWNLOADSKIPPED:           str = "DI_DOWNLOADSKIPPED"; break; 
    case DI_EFFECTRESTARTED:           str = "TDI_EFFECTRESTARTED"; break; 
    case DI_POLLEDDEVICE:              str = "DI_POLLEDDEVICE"; break;
    case DI_TRUNCATED:                 str = "DI_TRUNCATED"; break;
    case DI_TRUNCATEDANDRESTARTED:     str = "DI_TRUNCATEDANDRESTARTED"; break;
    case DIERR_ACQUIRED:               str = "DIERR_ACQUIRED"; break;
    case DIERR_ALREADYINITIALIZED:     str = "DIERR_ALREADYINITIALIZED"; break;
    case DIERR_BADDRIVERVER:           str = "DIERR_BADDRIVERVER"; break;
    case DIERR_BETADIRECTINPUTVERSION: str = "DIERR_BETADIRECTINPUTVERSION"; break;
    case DIERR_DEVICEFULL:             str = "DIERR_DEVICEFULL"; break;
    case DIERR_DEVICENOTREG:           str = "DIERR_DEVICENOTREG"; break;
    case DIERR_EFFECTPLAYING:          str = "DIERR_EFFECTPLAYING"; break;
    case DIERR_HASEFFECTS:             str = "DIERR_HASEFFECTS"; break;
    case DIERR_GENERIC:                str = "DIERR_GENERIC"; break;
    case DIERR_HANDLEEXISTS:           str = "DIERR_HANDLEEXISTS"; break; 
    case DIERR_INCOMPLETEEFFECT:       str = "DIERR_INCOMPLETEEFFECT"; break;
    case DIERR_INPUTLOST:              str = "DIERR_INPUTLOST"; break;
    case DIERR_INVALIDPARAM:           str = "ADIERR_INVALIDPARAM"; break;
    case DIERR_MOREDATA:               str = "DIERR_MOREDATA"; break;
    case DIERR_NOAGGREGATION:          str = "DIERR_NOAGGREGATION"; break;
    case DIERR_NOINTERFACE:            str = "DIERR_NOINTERFACE"; break;
    case DIERR_NOTACQUIRED:            str = "DIERR_NOTACQUIRED"; break;
    case DIERR_NOTBUFFERED:            str = "DIERR_NOTBUFFERED"; break;
    case DIERR_NOTDOWNLOADED:          str = "DIERR_NOTDOWNLOADED"; break;
    case DIERR_NOTEXCLUSIVEACQUIRED:   str = "DIERR_NOTEXCLUSIVEACQUIRED"; break;
    case DIERR_NOTFOUND:               str = "DIERR_NOTFOUND"; break;
    case DIERR_NOTINITIALIZED:         str = "DIERR_NOTINITIALIZED"; break;
    case DIERR_OLDDIRECTINPUTVERSION:  str = "DIERR_OLDDIRECTINPUTVERSION"; break;   
    case DIERR_OUTOFMEMORY:            str = "DIERR_OUTOFMEMORY"; break;
    case DIERR_UNSUPPORTED:            str = "DIERR_UNSUPPORTED"; break;
    case E_PENDING:                    str = "E_PENDING"; break;
    default: str = "UNKNOWN ERROR ["+IntToStr(Error)+"]";
    }
return str;
}
// --------------------------------------------------------------------------
// Func:            TDx_Input_Libray_ErrorMessage()
// Description:     Use this function to return information about the error.
// --------------------------------------------------------------------------
AnsiString __fastcall TDx_Input_Library_ErrorMessage(int Error)
{
AnsiString str = TDx_Library_ErrorMessage(Error);
if (str!="") return str;
switch (Error)
    {
    case -1: str = "You have attempted to call TDx_Input_Library component function before the component has been created. Refer to the help on the various TDx_Input_Library components Create() & Destroy() functions."; break;
    case DI_OK:	                       str = "The request completed successfully"; break;
    case DI_BUFFEROVERFLOW:            str = "The device buffer overflowed and some input was lost. This value is equal to the S_FALSE standard COM return value."; break; 
    case DI_DOWNLOADSKIPPED:           str = "The parameters of the effect were successfully updated, but the effect could not be downloaded because the associated device was not acquired in exclusive mode."; break; 
    case DI_EFFECTRESTARTED:           str = "The effect was stopped, the parameters were updated, and the effect was restarted."; break; 
    case DI_POLLEDDEVICE:              str = "The device is a polled device. As a result, device buffering will not collect any data and event notifications will not be signaled until the IDirectInputDevice2::Poll method is called. "; break;
    case DI_TRUNCATED:                 str = "The parameters of the effect were successfully updated, but some of them were beyond the capabilities of the device and were truncated to the nearest supported value. "; break;
    case DI_TRUNCATEDANDRESTARTED:     str = "Equal to DI_EFFECTRESTARTED | DI_TRUNCATED."; break;
    case DIERR_ACQUIRED:               str = "The operation cannot be performed while the device is acquired. "; break;
    case DIERR_ALREADYINITIALIZED:     str = "This object is already initialized "; break;
    case DIERR_BADDRIVERVER:           str = "The object could not be created due to an incompatible driver version or mismatched or incomplete driver components. "; break;
    case DIERR_BETADIRECTINPUTVERSION: str = "The application was written for an unsupported prerelease version of DirectInput. "; break;
    case DIERR_DEVICEFULL:             str = "The device is full. "; break;
    case DIERR_DEVICENOTREG:           str = "The device or device instance is not registered with DirectInput. This value is equal to the REGDB_E_CLASSNOTREG standard COM return value. "; break;
    case DIERR_EFFECTPLAYING:          str = "The parameters were updated in memory but were not downloaded to the device because the device does not support updating an effect while it is still playing. "; break;
    case DIERR_HASEFFECTS:             str = "The device cannot be reinitialized because there are still effects attached to it."; break;
    case DIERR_GENERIC:                str = "An undetermined error occurred inside the DirectInput subsystem. This value is equal to the case E_FAIL standard COM return value. "; break;
    case DIERR_HANDLEEXISTS:           str = "The device already has an event notification associated with it. This value is equal to the case E_ACCESSDENIED standard COM return value."; break; 
    case DIERR_INCOMPLETEEFFECT:       str = "The effect could not be downloaded because essential information is missing. For example, no axes have been associated with the effect, or no type-specific information has been supplied."; break;
    case DIERR_INPUTLOST:              str = "Access to the input device has been lost. It must be reacquired. "; break;
    case DIERR_INVALIDPARAM:           str = "An invalid parameter was passed to the returning function, or the object was not in a state that permitted the function to be called. This value is equal to the E_INVALIDARG standard COM return value. "; break;
    case DIERR_MOREDATA:               str = "Not all the requested information fitted into the buffer."; break;
    case DIERR_NOAGGREGATION:          str = "This object does not support aggregation. "; break;
    case DIERR_NOINTERFACE:            str = "The specified interface is not supported by the object. This value is equal to the E_NOINTERFACE standard COM return value. "; break;
    case DIERR_NOTACQUIRED:            str = "The operation cannot be performed unless the device is acquired. "; break;
    case DIERR_NOTBUFFERED:            str = "The device is not buffered. Set the DIPROP_BUFFERSIZE property to enable buffering."; break;
    case DIERR_NOTDOWNLOADED:          str = "The effect is not downloaded."; break;
    case DIERR_NOTEXCLUSIVEACQUIRED:   str = "The operation cannot be performed unless the device is acquired in DISCL_EXCLUSIVE mode. "; break;
    case DIERR_NOTFOUND:               str = "The requested object does not exist. "; break;
    case DIERR_NOTINITIALIZED:         str = "This object has not been initialized. "; break;
    case DIERR_OLDDIRECTINPUTVERSION:  str = "The application requires a newer version of DirectInput. "; break;
    case DIERR_OUTOFMEMORY:            str = "The DirectInput subsystem couldn't allocate sufficient memory to complete the call. This value is equal to the E_OUTOFMEMORY standard COM return value. "; break;
    case DIERR_UNSUPPORTED:            str = "The function called is not supported at this time. This value is equal to the E_NOTIMPL standard COM return value. "; break;
    case E_PENDING:                    str = "Data is not yet available."; break;
    default: str = "UNKNOWN ERROR ["+IntToStr(Error)+"]";
    }
return str;
}

//---------------------------------------------------------------------------
